function obj = fellipse(xcontact)

%===================
% Solve the Laplace--Young equation
% and compute the objective function (obj)
% defined with respect to the contact angle
%
% LEGEND:
%
% xcontact: elevation of nodes around the contact angle
% obj:      contact angle objective function
%===================

global A
global Nxi Net
global xi0 ximax snhxi0 cshxi0 
global xi cshxi snhxi
global eta cseta sneta
global Niter tolit
global capls
global x y z J
global forcex forcey forcez torqux torquy torquz
global cang
global cangle

%---
% prepare
%---

Dxi = (ximax-xi0)/Nxi;
Dxi2 = 2.0*Dxi;

Deta  = 2.0*pi/Net;
Deta2 = 2.0*Deta;

%---
% boundary values for i=1 (around the ellipse)
%---

 for j=1:Net
  x(1,j) = xcontact(j);
 end

%---
% wrap
%---

 x(1,Net+1) = x(1,1);
 x(1,Net+2) = x(1,2);

%----
% prepare for iterations
%----

 nonlinear = 0;
 omega = 0.5;
 Iflag = 0;

%======
for iterations=1:Niter
%======

%---
% compute the first derivatives
%---

for i=2:Nxi

 for j=2:Net+1
  MAT(1,1) =  A*cshxi(i)*cseta(j);
  MAT(1,2) =  A*snhxi(i)*sneta(j);
  MAT(2,1) = -MAT(1,2);
  MAT(2,2) =  MAT(1,1);
  RHS(1) = (x(i+1,j)-x(i-1,j))/Dxi2;
  RHS(2) = (x(i,j+1)-x(i,j-1))/Deta2;
  SOL = RHS/MAT';
  dxdy(i,j) = SOL(1);
  dxdz(i,j) = SOL(2);
 end

 % wrap:

 dxdy(i,1) = dxdy(i,Net+1);
 dxdz(i,1) = dxdz(i,Net+1);
 dxdy(i,Net+2) = dxdy(i,2);
 dxdz(i,Net+2) = dxdz(i,2);

end

% boundary values needed for the second derivative

for j=1:Net+2

  dxdy(1,j)    = 2.0*dxdy(2,j) -dxdy(3,j);
  dxdz(1,j)    = 2.0*dxdz(2,j) -dxdz(3,j);

  dxdy(1,j) = 3.0*dxdy(2,j)-3.0*dxdy(3,j)+dxdy(4,j);
  dxdz(1,j) = 3.0*dxdz(2,j)-3.0*dxdz(3,j)+dxdz(4,j);

  dxdy(Nxi+1,j) = 2.0*dxdy(Nxi,j)-dxdy(Nxi-1,j);
  dxdz(Nxi+1,j) = 2.0*dxdz(Nxi,j)-dxdz(Nxi-1,j);

  dxdy(Nxi+1,j) = 3.0*dxdy(Nxi,j)-3.0*dxdy(Nxi-1,j)+dxdy(Nxi-2,j);
  dxdz(Nxi+1,j) = 3.0*dxdz(Nxi,j)-3.0*dxdz(Nxi-1,j)+dxdz(Nxi-2,j);

end

%---
% compute the second derivatives
%---

for i=2:Nxi
 for j=2:Net+1
  MAT(1,1) =  A*cshxi(i)*cseta(j);
  MAT(1,2) =  A*snhxi(i)*sneta(j);
  MAT(2,1) = -MAT(1,2);
  MAT(2,2) =  MAT(1,1);
  RHS(1) = (dxdy(i+1,j)-dxdy(i-1,j))/Dxi2;
  RHS(2) = (dxdy(i,j+1)-dxdy(i,j-1))/Deta2;
  SOL = RHS/MAT';
  dxdyy(i,j) = SOL(1);
  dxdyz(i,j) = SOL(2);
  RHS(1) = (dxdz(i+1,j)-dxdz(i-1,j))/Dxi2;
  RHS(2) = (dxdz(i,j+1)-dxdz(i,j-1))/Deta2;
  SOL = RHS/MAT';
  dxdzy(i,j) = SOL(1);
  dxdzz(i,j) = SOL(2);

  %---
  % optional, to test the laplacian
  %---

  JS = J(i,j)*J(i,j);
  laplacian1(i,j) = ((x(i+1,j)-2*x(i,j)+x(i-1,j))/Dxi^2 ...
            + (x(i,j+1)-2*x(i,j)+x(i,j-1))/Deta^2)/JS;
  %---

% [dxdyy dxdyz dxdzy dxdzz]
% pause

 end
end

%dxdy'
%dxdz'
%dxdyy'
%dxdyz'
%dxdzy'
%dxdzz'
%x'
%pause

% test the laplacian

%laplacian = dxdyy+dxdzz;
%aa = laplacian (2,:);
%bb = laplacian1(2,:);
%[aa;bb]'
%pause

%------
% perform iterations by
% scanning the grid points
%------

err = 0.0;

xneo = x;

for i=2:Nxi
 for j=2:Net+1
  JS = J(i,j)*J(i,j);
  xnew = ((x(i+1,j)+x(i-1,j))/Dxi^2 ...
        + (x(i,j+1)+x(i,j-1))/Deta^2)/JS;
%~~~
  if(nonlinear==1)
   xnew = xnew +  dxdz(i,j)^2               *dxdyy(i,j);
%  xnew = xnew - 2.0*dxdy(i,j)*dxdz(i,j)    *dxdyz(i,j);
   xnew = xnew - dxdy(i,j)*dxdz(i,j)   * (dxdyz(i,j) + dxdzy(i,j));
   xnew = xnew +  dxdy(i,j)^2               *dxdzz(i,j);
  end
%~~~

  tmp = 1.0+dxdy(i,j)^2+dxdz(i,j)^2;
  G = 2.0*(1.0/Dxi^2+1.0/Deta^2)/JS + tmp^(3/2)/capls;

  xnew = xnew/G;

  diff = abs(xnew-x(i,j));

  if(diff>err) err = diff; end

  xneo(i,j) = x(i,j)+omega*(xnew-x(i,j));

 end
end

% err

 x = xneo;

% wrap:

 for i=2:Nxi+1
  x(i,1)     = x(i,Net+1);
  x(i,Net+2) = x(i,2);
 end

%---
% stopping check
%---

if(err<10.0*tolit)
  nonlinear = 1;
  omega = 0.2;
end

if(err<tolit)
  Iflag = 1;
  break;
end

%======
end % of iterations
%======

if(Iflag == 0)
 disp('iterations did not coverge')
 pause
end

%============
% compute dxdxi, dxdeta,
% the surface normal and tension vector
% around the contact line
%============

x0 = 0.0;
y0 = 0.0;
z0 = 0.0;

for j=1:Net+1

 dxdxi(j) = (-x(3,j)+4.0*x(2,j)-3.0*x(1,j))/Dxi2;
 dydxi(j) = A*cshxi0*cseta(j);
 dzdxi(j) = A*snhxi0*sneta(j);
 dmdxi(j) = sqrt(dxdxi(j)^2+dydxi(j)^2+dzdxi(j)^2);

 if(j==1)
  dxdeta(j) = (x(1,2)-x(1,Net))/Deta2;
 else
  dxdeta(j) = (x(1,j+1)-x(1,j-1))/Deta2;
 end

 dydeta(j) =-dzdxi(j);
 dzdeta(j) = dydxi(j);
 dmdeta(j) = sqrt(dxdeta(j)^2+dydeta(j)^2+dzdeta(j)^2);

 % cross product and normal vector

 vnx(j) = dydxi(j)*dzdeta(j)-dzdxi(j)*dydeta(j);
 vny(j) = dzdxi(j)*dxdeta(j)-dxdxi(j)*dzdeta(j);
 vnz(j) = dxdxi(j)*dydeta(j)-dydxi(j)*dxdeta(j);
 vnm(j) = sqrt(vnx(j)^2+ vny(j)^2+vnz(j)^2);
 vnx(j) = vnx(j)/vnm(j);
 vny(j) = vny(j)/vnm(j);
 vnz(j) = vnz(j)/vnm(j);

 % tangential vector

 tngx(j) = dydeta(j)*vnz(j)-dzdeta(j)*vny(j);
 tngy(j) = dzdeta(j)*vnx(j)-dxdeta(j)*vnz(j);
 tngz(j) = dxdeta(j)*vny(j)-dydeta(j)*vnx(j);

 % cross of tangential vector with position vector

 crsx(j) = (y(1,j)-y0)*tngz(j)-(z(1,j)-z0)*tngy(j);
 crsy(j) = (z(1,j)-z0)*tngx(j)-(x(1,j)-x0)*tngz(j);
 crsz(j) = (x(1,j)-x0)*tngy(j)-(y(1,j)-y0)*tngx(j);

end

%---
% cylinder normal vector
%---

for j=1:Net+1
 vnpx(j) = 0.0;
 vnpy(j) = A*cshxi(1)*cseta(j);
 vnpz(j) = A*snhxi(1)*sneta(j);
 vnpm(j) = sqrt(vnpx(j)^2+vnpy(j)^2+vnpz(j)^2);
 vnpx(j) = vnpx(j)/vnpm(j);
 vnpy(j) = vnpy(j)/vnpm(j);
 vnpz(j) = vnpz(j)/vnpm(j);
end

%---
% contact angle
%---

for j=1:Net+1
  prj = vnx(j)*vnpx(j) ...
       +vny(j)*vnpy(j) ...
       +vnz(j)*vnpz(j);
  cangle(j) = acos(prj);
end

%---
% object function
%---

for j=1:Net
  obj(j) = cangle(j)-cang;
end

%---
% force and torque
%---

forcex = 0.0;
forcey = 0.0;
forcez = 0.0;
torqux = 0.0;
torquy = 0.0;
torquz = 0.0;

for j=1:Net
  forcex = forcex+tngx(j);
  forcey = forcey+tngy(j);
  forcez = forcez+tngz(j);
  torqux = torqux+crsx(j);
  torquy = torquy+crsy(j);
  torquz = torquz+crsz(j);
end

forcex = forcex*Deta;
forcey = forcey*Deta;
forcez = forcez*Deta;

torqux = torqux*Deta;
torquy = torquy*Deta;
torquz = torquz*Deta;

%[forcex, forcey, forcez]
%[torqux, torquy, torquz]

%---
% done
%---

return
