      subroutine films
     +
     +    (NLR
     +    ,thick
     +    ,visc
     +    ,den
     +    ,gac
     +    ,beta
     +    ,y
     +    ,yI,uI
     +    ,Q
     +    ,u
     +    )

c=========================================
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved
c
c This program is to be used only under the
c stipulations of the licensing agreement
c=========================================

c=========================================
c Multi-film flow down an inclined plane
c located at y=0
c
c Film numbered 1 is adjacent to the plane
c
c The velocity in the ith film is given by:
c
c u(i) = af(i) + bf(i)*y - gf(i)*y**2
c
c SYMBOLS:
c -------
c
c NLR:    Number of films
c yI(i):  y-position of the ith interface
c uI(i):  velocity at the ith interface
c Q(i);   flow rate of ith film
c beta:   plane inclination angle
c gac:    magnitude of the acceleration of gravity
c=========================================

      Implicit Double Precision (a-h,o-z)

      Dimension thick(10),visc(10),den(10)
      Dimension yI(10),uI(10),Q(10)
      Dimension gf(10),af(10),bf(10)

c----------
c constants
c----------

      pi = 3.14159 265358 D0

c----------------------
c set the interface positions
c----------------------

      yI(1) = thick(1)

      Do i=2,NLR
       yI(i) = yI(i-1)+thick(i)
      End Do

c-----------------------------------------
c The velocity in the ith film is given by:
c
c u(i) = af(i) + bf(i)*y - gf(i)*y**2
c
c compute gf(i)
c-----------------------------------------

      fc = 0.5D0*gac*Dsin(beta)

      Do i=1,NLR
        gf(i) = fc*den(i)/visc(i)
      End Do

c---
c compute bf(i) by recursion
c starting from the top
c and requiring continuity of shear stress
c---

      bf(NLR) = 2.0D0*gf(NLR)*yI(NLR)

      Do i=NLR-1,1,-1
        i1 = i+1
        bf(i) = 2.0*gf(i)*yI(i) 
     +        + visc(i1)/visc(i)*(bf(i1)-2.0*gf(i1)*yI(i)) 
      End Do

c---
c compute af(i) by recursion
c starting from the bottom
c and requiring continuity of velocity
c---

      af(1) = 0.0

      Do i=1,NLR-1
       i1 = i+1
       af(i1) = af(i)+( (bf(i)-bf(i1)) 
     +                 -(gf(i)-gf(i1))*yI(i)) * yI(i)
      End Do

c---
c compute the interfacial velocities
c---

      Do i=1,NLR
       uI(i) = af(i) + (bf(i)-gf(i)*yI(i)) * yI(i)
      End Do

c----
c compute the flow rates in the ith film
c is confined between i-1 and i interfaces
c----

      yI0 = 0.0D0;
      Q(1) = af(1)*(yI(1)-yI0)
     +      +bf(1)*(yI(1)**2-yI0**2)/2.0D0
     +      -gf(1)*(yI(1)**3-yI0**3)/3.0D0


      Do i=2,NLR
       Q(i) = af(i)*(yI(i)   -yI(i-1))
     +       +bf(i)*(yI(i)**2-yI(i-1)**2)/2.0
     +       -gf(i)*(yI(i)**3-yI(i-1)**3)/3.0
      End Do

c--------------------------
c compute the velocity at y
c--------------------------

c---
c identify the host layer
c numbered j
c---

      i = 1
 92   Continue

      if(y.gt.yI(i)) then
       i = i+1
       Go to 92
      end if

      if(i.gt.NLR) i = NLR
      j = i
     
c---
c evaluate the velocity
c---

      u = af(j)+(bf(j)-gf(j)*y)*y

c-----
c Done
c-----

      Return
      End
