      program films_dr

c=========================================
c Copyright by C. Pozrikidis, 1999
c All rights reserved
c
c This program is to be used only under the
c stipulations of the licencing agreement
c=========================================

c===============================================
c Multi-film flow down an inclined plane
c
c Film numbered 1 is adjacent to the plane
c FIlm numbered NLR is adjacent to the free surface
c
c The velocity profile within the ith film is given by:
c
c u(i) = af(i) + bf(i)*y - gf(i)*y**2
c
c where the inclined plane is located at y = 0.
c
c SYMBOLS:
c -------
c
c NLR:    Number of films
c thick(i): thickness of the ith layer
c yI(i):  y-position of the ith interface
c uI(i):  velocity at the ith interface
c Q(i):   flow rate of ith film
c beta:   plane inclination angle
c gac:    magnitude of the acceleration of gravity
c nprof:  number of points per layer for a profile
c
c===============================================

      Implicit Double Precision (a-h,o-z)

      Dimension thick(10),visc(10),den(10)
      Dimension yI(10),uI(10),Q(10)

c---
c constants
c---

      pi = 3.14159265358D0

      Null    = 0
      None    = 1
      Ntwo    = 2
      Nthree  = 3
      Nfour   = 4
      Nfive   = 5

      zero    = 0.0

c---------
c  input
c---------

      open (1,file="films.dat",status="unknown")

        read (1,*) nprof
        read (1,*) beta
        read (1,*) gac
        read (1,*)
        read (1,*) NLR
        read (1,*) (thick(i),i=1,NLR)
        read (1,*) (visc (i),i=1,NLR)
        read (1,*) (den  (i),i=1,NLR)

      close (1)

c---
c prepare
c---

      open (2,file="films.out",status="unknown")
      write (2,900) NLR

      beta = beta*pi

      cth0 = cos(beta)
      sth0 = sin(beta)

c---
c compute the total thickness
c---

      h = 0.0D0      ! total film thickness

      Do i=1,NLR
        h = h+thick(i)
      End Do

c interface positions

      yI(1) = thick(1)

      Do i=2,NLR
       yI(i) = yI(i-1)+thick(i)
      End Do

c------------------------
c Draw the inclined plane
c------------------------

      write (2,900) Nfive

      xx = -h
      yy = 0.0
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) None,xprint,yprint

      xx = h
      yy = 0.0
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) Ntwo,xprint,yprint

      xx =  h
      yy = -0.2*h
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) Nthree,xprint,yprint

      xx = -h
      yy = -0.2*h
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) Nfour,xprint,yprint

      xx = -h
      yy = 0.0
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) Nfive,xprint,yprint

c-- inclined plane

      write (2,900) Ntwo

      xx = -h
      yy = zero
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) None,xprint,yprint

      xx =  h
      yy = zero
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) Ntwo,xprint,yprint

c---
c draw the interfaces
c---

      Do i=1,NLR

       write (2,900) Ntwo
       xx = -h
       yy = yI(i)
       xprint = xx*cth0+yy*sth0
       yprint =-xx*sth0+yy*cth0

       write (2,900) None,xprint,yprint
       xx = h
       yy = yI(i)
       xprint = xx*cth0+yy*sth0
       yprint =-xx*sth0+yy*cth0
       write (2,900) Ntwo,xprint,yprint

      End Do

c---
c draw a vertical line
c---

      write (2,900) Ntwo

      xx = 0.0
      yy = 0.0
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) None,xprint,yprint

      xx = 0.0
      yy = 1.3*h
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) Ntwo,xprint,yprint

c---
c draw the velocity profile
c---
     
      nproftot = NLR*nprof+1    ! total number of points for profile

      write (2,900) nproftot

      Ic = 1           ! counter
      y  = 0
      u  = 0

c     write (6,*)
c     write (6,*)  " velocity profile"
c     write (6,*)

      write (2,900) ic,u,y
c     write (6,900) ic,u,y

      Do i=1,NLR

       step = thick(i)/(nprof+1.0D0-1.0D0)

       Do j=1,nprof
         y  = y+step
         Ic = Ic+1

c        y = 0.234395; % testing

         call films
     +
     +    (NLR
     +    ,thick
     +    ,visc
     +    ,den
     +    ,gac
     +    ,beta
     +    ,y
     +    ,yI
     +    ,uI
     +    ,Q
     +    ,u
     +    )

c      write (6,*) u
c      pause

       xx = u
       yy = y

       xprint = xx*cth0+yy*sth0
       yprint =-xx*sth0+yy*cth0

       write (2,900) Ic,xprint,yprint
c      write (6,900) Ic,xprint,yprint

       End Do

      End Do

c---
c end of drawing
c---

      write (2,*) null
      close (2)

c---
c printing session
c---

      write (6,*)
      write (6,*) " Interface y position, velocity"
      write (6,*)

      Do i=1,NLR
       write (6,501) i,yI(i),uI(i)
      End Do

      write (6,*)
      write (6,*) " Layer flow rates and mean velocity"
      write (6,*)

      Do i=1,NLR
       umean = Q(i)/(yI(i)-yI(i-1))
       write (6,900) i,Q(i),umean
      End Do

c---
c Done
c---

  99  Continue


 501  Format ("Location and velocity of interface",i3," : "
     +       ,f15.10,2x,f15.10)
 900  Format (1x,i3,4(1x,f10.5))

      Stop
      End
