function [sI,uI,xi1,xi2,Q,u] = tube_ann_sw_ml ...
...
        (NLR ...
        ,a1 ...
        ,a2 ...
        ,thick ...
        ,visc ...
        ,Omega1 ...
        ,Omega2 ...
        ,s)

%===========================================
% Multi-layer swirling flow inside an annular tube
%
% Layer numbered 1 is adjacent to the inner cylinder
% Layer numbered NLR is adjacent to outer cylinder
%
% The velocity in the ith film is given by:
%
% u(i) = bf(i)/sigma + cf(i)*sigma
%
% SYMBOLS
% -------
%
% a1:        inner tube radius
% a2:        outer tube radius
% Omega1:    inner tube angular velocity
% Omega2:    outer tube angular velocity
% NLR:       Number of layers
% thick(i):  thickness of the ith layer
% sI(i):     radial position of the ith interface
% uI(i):     velocity of the ith interface
% xi1(i):    shear rate on lower side of the ith interface
% xi2(i):    shear rate on upper side of the ith interface
% Q(i):      flow rate of ith layer
%===========================================

%---
% set the radial interface positions
%---

sI(1) = a1+thick(1);

for i=2:NLR
  sI(i) = sI(i-1)+thick(i);
end

%-----------------------------------------
% The velocity in the ith layer is given by:
%
% u(i) =  bf(i)/sigma + cf(i)*sigma
%
% In the first pass, set bf(1)=0
% and compute the residual of the inner cylinder boundary
% condition: error0
%
% In the second pass, set bf(1)=1
% and compute the residual of the inner cylinder boundary
% condition: error1
%
% In the third pass, set the proper value:
%
% bf(1) = error0/(error0-error1)
%-----------------------------------------

bf(1) = 0.0;

%---
for Ipass=1:3
%---

%---
% compute bf(i) by recursion
% starting from the inner cylinder by requiring
% continuity of shear stress across the interfaces
%
% The sigma-phi shear stress is given by:
%
% shear_stress = sigma d(u/sigma)/d(sigma)
%---

  for i=1:NLR-1
    bf(i+1) = visc(i)/visc(i+1)*bf(i);
  end

%---
% compute cf(i) by recursion
% to satisfy continuity of velocity across the iterfaces
%---

   cf(1) = (Omega1*a1-bf(1)/a1)/a1;

   for i=1:NLR-1
      cf(i+1) = cf(i) + (bf(i)-bf(i+1))/sI(i)^2;
   end

%---
% error in the no-slip condition on the inner wall
% error = u(s=a1)-V1
%---

      if(Ipass==1)
        error0 = bf(NLR)/a2 + cf(NLR)*a2 - Omega2*a2;
        bf(1)  = 1.0;
      elseif(Ipass==2)
        error1 = bf(NLR)/a2 + cf(NLR)*a2 - Omega2*a2;
        bf(1)  = error0/(error0-error1);
      end

%---
      end  % of Ipass
%---

%---
% compute interfacial velocities
% and shear rates
%---

    for i=1:NLR-1
       uI(i) = bf(i)/sI(i) + cf(i)*sI(i);
       xi1(i) = -2.0*bf(i) /sI(i)^3;
       xi2(i) = -2.0*bf(i+1)/sI(i)^3; 
    end

%--------------------------------------------------
% compute the flow rates
% ith film is confined between i-1 and i interfaces
%--------------------------------------------------

      Q(1) = bf(1)*(log(sI(1)/a1)) ...
           + cf(1)*0.5D0*(sI(1)^2-a1^2);

      for i=2:NLR
       Q(i) = bf(i)*log(sI(i)/sI(i-1)) ...
            + cf(i)*0.5*(sI(i)^2-sI(i-1)^2);
      end

%--------------------------
% compute the velocity at s
%--------------------------

%---
% identify the host layer
%---

 for i=1:NLR
   if(s<sI(i))
     break
   end
 end

 if(i>NLR) i=NLR; end

%---
% evaluate the velocity at s
%---

u  = bf(i)/s+cf(i)*s;

%-----
% Done
%-----

return
