function [sI,uI,xi1,xi2,Q,u] = tube_crc_ml ...
...
    (NLR ...
    ,a ...
    ,thick ...
    ,visc ...
    ,den ...
    ,gac ...
    ,chi ...
    ,beta ...
    ,s...
    )

%=============================================
% Multi-layer core-annular flow through a circular tube
%
% Film numbered 1 is at the centerline
% Film numbered NLR is adjacent to the tube surface
%
% The velocity in the ith film is given by:
%
% u(i) = af(i) + bf(i)*ln(sigma/a) - gf(i)*sigma^2
%
% SYMBOLS
% ------
%
% NLR:       Number of layers
% a:         tube radius
% thick(i):  thickness of the ith layer
% sI(i):     position of the ith interface
% uI(i):     velocity of the ith interface
% xi1(i):    shear rate on lower side of the ith interface
% xi2(i):    shear rate on upper side of the ith interface
% Q(i):      flow rate of ith layer
% beta:      tube inclination angle
% gac:       magnitude of the acceleration of gravity
% chi:       negative of the pressure gradient
%=============================================

%---
% set the interface positions
%---

sI(1) = thick(1);

for i=2:NLR
  sI(i) = sI(i-1)+thick(i);
end

%-----------------------------------------
% The velocity in the ith layer is given by:
%
% u(i) = af(i) + bf(i)*ln(sigma/a) - gf(i)*sigma^2
%
% compute gf(i)
%-----------------------------------------

fc = gac*sin(beta);

for i=1:NLR
  gf(i) = 0.25*(chi+fc*den(i))/visc(i);
end

%---
% compute bf(i) by recursion
% starting from the centerline and
% by requiring continuity of shear stress
% across the interfaces
%---

bf(1) = 0.0;

for i=2:NLR
   bf(i) =  2.0*gf(i)*sI(i-1)^2 ...
          + visc(i-1)/visc(i) * (bf(i-1)-2.0*gf(i-1)*sI(i-1)^2); 
end

%---
% compute af(i) by recursion
% starting from the tube wall and
% and requiring continuity of velocity
%---

af(NLR) = gf(NLR)*a^2;

for i=NLR-1:-1:1
  af(i) = af(i+1)+(bf(i+1)-bf(i))*log(sI(i)/a) ...
               -(gf(i+1)-gf(i))*sI(i)^2;
end

%---
% compute interfacial velocities
% and shear rates
%---

for i=1:NLR-1
  uI(i) = af(i) + bf(i)*log(sI(i)/a) - gf(i)*sI(i)^2;
  xi1(i) = bf(i)  /sI(i) - 2.0*gf(i)  *sI(i);
  xi2(i) = bf(i+1)/sI(i) - 2.0*gf(i+1)*sI(i);
end

%----
% compute the flow rates
% the ith film is confined between i-1 and i interfaces
%----

      sI1s = sI(1)^2;
      Q(1) = af(1)*sI1s/2.0 ...
           + bf(1)*0.5*sI1s*(log(sI(1)/a)-0.5) ...
            -gf(1)*sI1s^2/4.0;
      Q(1) = 2.0*pi*Q(1);

      for i=2:NLR
       sIis  = sI(i)^2;
       sIias = sI(i-1)^2;
       Q(i) = af(i)*(sIis-sIias)/2.0 ...
             +bf(i)*0.5*(sIis *(log(sI(i)/a)-0.5D0) ...
                        -sIias*(log(sI(i-1)/a)-0.5D0)) ...
             -gf(i)*(sIis^2-sIias^2)/4.0;
       Q(i) = 2.0*pi*Q(i);
      end

%--------------------------
% compute the velocity at s
%--------------------------

% identify the host layer

 for i=1:NLR
   if(s<sI(i)) break; end
 end

 if(i>NLR) i = NLR; end

%---
% evaluate the velocity at s
%---

  if(s>0.000001)
   u = af(i) + bf(i)*log(s/a) - gf(i)*s^2;
  else
   u = af(i)-gf(i)*s^2;
  end

%-----
% Done
%-----

   return
