      program tube_crc_ml_dr

c============================================
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved.
c
c This program is to be used only under the
c stipulations of the licencing agreement
c============================================

c=====================================================
c Multi-layer core-annular flow through a circular tube
c
c Film numbered 1 is at the centerline
c Film numbered NLR is adjacent to the the wall
c
c The velocity in the ith film is given by:
c
c u(i) = af(i) + bf(i)*ln(sigma/a) - gf(i)*sigma**2
c
c SYMBOLS
c ------
c
c NLR:       Number of films
c thick(i):  thickness of the ith layer
c sI(i):     sigma-position of the ith interface
c uI(i):     velocity of the ith interface
c xi1(i):    shear rate on lower side of the ith interface
c xi2(i):    shear rate on upper side of the ith interface
c Q(i):      flow rate of ith layer
c beta:      inclination angle
c gac:       magnitude of the acceleration of gravity
c chi:       negative of the pressure gradient
c nprof:     number of points per layer for a profile
c=====================================================

      Implicit Double Precision (a-h,o-z)

      Dimension thick(10),visc(10),den(10)
      Dimension sI(10),uI(10)
      Dimension xi1(10),xi2(10),Q(10)

      Dimension xxs(1000),yys(1000)

c---
c constants
c---

      pi = 3.14159 265358 D0

      Null   = 0
      None   = 1
      Ntwo   = 2
      Nthree = 3
      Nfour  = 4
      Nfive  = 5

      zero = 0.0

c-------
c  input
c-------

      open (1,file="tube_crc_ml.dat",status="unknown")

        read (1,*) nprof   ! number of points per layer for a profile
        read (1,*) beta
        read (1,*) gac
        read (1,*) a
        read (1,*) chi
        read (1,*)
        read (1,*) NLR
        read (1,*) (thick(i),i=1,NLR-1)
        read (1,*) (visc (i),i=1,NLR)
        read (1,*) (den  (i),i=1,NLR)

      close (1)

c----------------------------
c check for consistency
c and set top-layer thickness
c----------------------------

      sum = 0.0D0

      Do i=1,NLR-1
        sum = sum+thick(i)
      End Do

      thick(NLR) = a-sum

      if(thick(NLR).lt.0) then
       write (6,*)
       write (6,*) " tube_crc_ml_dr: layer thicknesses are improper"
       stop
      end if

c---
c  interface positions
c---

      sI(1) = thick(1)

      Do i=2,NLR
       sI(i) = sI(i-1)+thick(i)
      End Do

c--------
c prepare
c--------

      nprof1 = nprof+1
      nproftot = NLR*nprof+1    ! total number of profile points

      beta = pi*beta

      cth0 = cos(beta)
      sth0 = sin(beta)

      open (2,file="tube_crc_ml.out",status="unknown")
      write (2,900) NLR

c-------------------
c paint the cylinder
c-------------------

      write (2,900) Nfive

      xx = -1.5*a
      yy =  a
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) None,xprint,yprint

      xx = 1.5*a
      yy = a
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) Ntwo,xprint,yprint

      xx = 1.5*a
      yy = 1.2*a
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) Nthree,xprint,yprint

      xx = -1.5*a
      yy =  1.2*a
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) Nfour,xprint,yprint

      xx = -1.5*a
      yy =  a
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) Nfive,xprint,yprint

c---------------------------
c reflection of the cylinder
c---------------------------

      write (2,900) Nfive

      xx = -1.5*a
      yy =  -a
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) None,xprint,yprint

      xx = 1.5*a
      yy = -a
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) Ntwo,xprint,yprint

      xx = 1.5*a
      yy =-1.2*a
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) Nthree,xprint,yprint

      xx = -1.5*a
      yy = -1.2*a
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) Nfour,xprint,yprint

      xx = -1.5*a
      yy = -a
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) Nfive,xprint,yprint

c---
c paint the layers
c---

c-----
c first layer
c-----

      write (2,900) Nfive

      xx = -a
      yy = zero
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) None,xprint,yprint

      xx = a
      yy = zero
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) Ntwo,xprint,yprint

      xx =  a
      yy = si(1) 
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) Nthree,xprint,yprint

      xx = -a
      yy = si(1)
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) Nfour,xprint,yprint

      xx = -a
      yy = zero
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) Nfive,xprint,yprint

c--- reflection

      write (2,900) Nfive

      xx = -a
      yy = zero
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) None,xprint,yprint

      xx = a
      yy = zero
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) Ntwo,xprint,yprint

      xx =  a
      yy =-si(1) 
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) Nthree,xprint,yprint

      xx = -a
      yy =-si(1)
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) Nfour,xprint,yprint

      xx = -a
      yy = zero
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) Nfive,xprint,yprint

c---
c subsequent layers
c---

      If(NLR.gt.1) then

      Do i=2,NLR

      write (2,900) Nfive

      xx = -a
      yy = si(i-1)
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) None,xprint,yprint

      xx = a
      yy = si(i-1)
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) Ntwo,xprint,yprint

      xx =  a
      yy = si(i) 
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) Nthree,xprint,yprint

      xx = -a
      yy = si(i)
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) Nfour,xprint,yprint

      xx = -a
      yy = si(i-1)
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) Nfive,xprint,yprint

c---
      write (2,900) Nfive

      xx = -a
      yy = -si(i-1)
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) None,xprint,yprint

      xx = a
      yy = -si(i-1)
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) Ntwo,xprint,yprint

      xx =  a
      yy =-si(i) 
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) Nthree,xprint,yprint

      xx = -a
      yy =-si(i)
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) Nfour,xprint,yprint

      xx = -a
      yy = -si(i-1)
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) Nfive,xprint,yprint

      End Do

      End If

c--------------------
c draw the centerline
c--------------------

      write (2,900) Ntwo

      xx = -1.2*a
      yy = 0.0
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) None,xprint,yprint

      xx = 1.2*a
      yy = zero
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) Ntwo,xprint,yprint

c---------------------
c draw a vertical line
c---------------------

      write (2,900) Ntwo

      xx = zero
      yy = -a
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) None,xprint,yprint

      xx = zero
      yy = a
      xprint = xx*cth0+yy*sth0
      yprint =-xx*sth0+yy*cth0
      write (2,900) Ntwo,xprint,yprint

c-----------------
c draw the profile
c-----------------

c     write (6,*)
c     write (6,*)  " velocity profile"
c     write (6,*)

      write (2,900) nproftot

      Ic = 1           ! counter

      s  = 0.0

      Do i=1,NLR

       step = thick(i)/(nprof1-1.0)

       max = nprof

       if(i.eq.NLR) max = nprof+1

       Do j=1,max

c        s = 0.2;

         call tube_crc_ml
     +
     +    (NLR
     +    ,a
     +    ,thick
     +    ,visc
     +    ,den
     +    ,gac
     +    ,chi
     +    ,beta
     +    ,s
     +    ,sI,uI
     +    ,xi1,xi2
     +    ,Q
     +    ,u
     +    )

c       write (6,*) u
c       pause

        xx = u
        yy = s

        xxs(Ic) = xx
        yys(Ic) = yy

        xprint = xx*cth0+yy*sth0
        yprint =-xx*sth0+yy*cth0

        write (2,900) Ic,xprint,yprint

        s = s+step

        Ic = Ic+1

       End Do
      End Do

c---------------------
c print the reflection
c---------------------

      write (2,900) nproftot

      Do i=1,nproftot

        xx = xxs(i)
        yy =-yys(i)

        xprint = xx*cth0+yy*sth0
        yprint =-xx*sth0+yy*cth0

        write (2,900) Ic,xprint,yprint

      End Do

c---------------
c end of drawing
c---------------

      write (2,*) null
      close (2)

c---
c printing session
c---

      write (6,*)
      write (6,*) " Interface s pos, vel, shear rates"
      write (6,*)

      Do i=1,NLR-1
       write (6,900) i,si(i),uI(i),xi1(i),xi2(i)
c      ratio = xi1(i)*visc(i)/(xi2(i)*visc(i+1))
c      write (6,*) ratio   ! should be unity
      End Do

      write (6,*)
      write (6,*) " Layer flow rates and mean velocity"
      write (6,*)

      Do i=1,NLR
       umean = Q(i)/pi/(sI(i)**2-sI(i-1)**2)
       write (6,900) i,Q(i),umean
      End Do

c---
c Done
c---

  99  Continue

 900  Format (1x,i3,9(1x,f15.10))

      Stop
      End
