      subroutine bear_2d_pressure 
     +
     + (visc
     + ,x
     + ,h
     + ,dhdx
     + ,Dx
     + ,V
     + ,N
     + ,p1,p2
     + ,pi1
     + ,pi2
     + )

c===================================
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved
c
c This program is to be used only under the
c stipulations of the licencing agreement
c===================================

c===================================
c Solve the lubrication equation for the pressure
c using a finite-difference method
c
c Solutions are generated for longitudinal motion (pi1)
c and normal motion (pi2)
c
c SYMBOLS:
c
c x(i):           grid nodes
c h(i):           gap
c visc:           viscosity
c p1:             left-end pressure
c p2:             right-end pressure
c act, btr, ctr:  tridiagonal entries
c rhs:            right-hand side
c sln:            solution for the pressure
c===================================

      Implicit Double Precision (a-h,o-z)

      Dimension x(200),h(200),dhdx(200)
      Dimension q(200),dqdx(200)

      Dimension atr(200),btr(200),ctr(200)
      Dimension rhs(200),sln(200)
      Dimension pi1(200),pi2(200)

c---
c prepare
c---

      Dxh = 0.5*Dx
      Dx2 = 2.0*Dx
      Dxs = Dx**2

c---
c auxiliary
c---

      Do i=1,N+1
        q(i) = h(i)**3/visc 
      End Do

      dqdx(1) = (q(2)-q(1))/Dx         ! forward difference

      Do i=2,N
        dqdx(i) = (q(i+1)-q(i-1))/Dx2  ! central difference
      End Do

      dqdx(N+1) = (q(N+1)-q(N))/Dx     ! backward difference

c---
c compute a particular solution
c with exact boundary conditions (p1 and p2)
c
c act, btr, ctr are the tridiagonal entries
c of the coefficient matrix
c---

      factor = 6.0*V*Dxs

      atr(1) = -2.0*q(2)
      tmp     =   dqdx(2)*Dxh
      btr(1) =      q(2)+tmp
      rhs(1) =  factor*dhdx(2) - p1*(q(2)-tmp)

      Do i=3,N-1
        atr(i-1) = -2.0*q(i)
        tmp    =   dqdx(i)*Dxh
        btr(i-1) =      q(i)+tmp
        ctr(i-1) =      q(i)-tmp
        rhs(i-1) =  factor*dhdx(i)
      End Do

      atr(N-1) = -2.0*q(N);
      ctr(N-1) =      q(N)-tmp;
      rhs(N-1) =  factor*dhdx(N)-p2*(q(N)+tmp);

c---
c solve N-1 equations by the Thomas algorithm
c---

      call thomas 
     +
     +   (N-1
     +   ,atr,btr,ctr
     +   ,rhs
     +   ,sln
     +   )

      pi1(1)   = p1     ! end node
      pi1(N+1) = p2     ! end node

      Do i=1,N-1
        pi1(i+1) = sln(i)
      End Do

c---
c compute a homogeneous solution with dcdt=1.0
c with homogeneous boundary conditions
c---

      factor = 12.0*Dxs 

      Do i=1,N-1
        rhs(i) = factor
      End Do

c---
c solve K-1 equations with the new rhs
c---

      call thomas 
     +
     +   (N-1
     +   ,atr,btr,ctr,rhs
     +   ,sln
     +   )

      pi2(1)   = 0.0    ! end node
      pi2(N+1) = 0.0    ! end node

      Do i=1,N-1
        pi2(i+1) = sln(i)
      End Do

c---
c Done
c---

 100  Format (1x,f15.10)

      Return
      End
