      program chan2l_exp

c=========================================
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved
c
c This program is to be used only under the
c stipulations of the licensing agreement
c=========================================

c=========================================
c Flow of two viscous layers in a horizontal or inclined
c channel confined between two parallel plane walls
c located at y=+-a
c
c Shear-, pressure-, and gravity-driven flow
c
c The program uses an explicit finite-difference method
c with uniform grid spacing.
c
c Legend:
c ------
c
c a:    walls are located at y = +-a
c beta: inclination angle
c RL:   period
c
c yIunp:  unperturbed position of the interface
c aI0:    initial amplitude of the interface
c asrf0:  initial amplitude of the surfactant
c
c vs1,vs2	layer viscosities
c rho1,rho2	layer densities
c
c vsr:		lamda (viscosity ratio)
c hr:      layer thickness ratio: h2/h1
c
c pdrop:  pressure drop over one period (positive)
c
c method: 1 for central differences
c         2 for backward differences
c
c DyDx1:  first derivative of y(x)
c DyDx2:  second derivative
c DyDx3:  third derivative
c
c================================================

      Implicit Double Precision (a-h,o-z)

      Dimension  x(-10:500), y(-10:500)
      Dimension h1(-10:500),h2(-10:500),hr(-10:500)
      Dimension  c(-10:500),DcDt(-10:500)
      Dimension srtn(-10:500),DsrtnDx(-10:500)

      Dimension DyDx1(-10:500),DyDx2(-10:500),DyDx3(-10:500)
      Dimension uI(-10:500)
      Dimension dp1dx(-10:500)
      Dimension store(-10:500),DyDt(-10:500)

      Dimension ysv(-10:500)
      Dimension csv(-10:500)
      Dimension DyDtsv(-10:500)
      Dimension DcDtsv(-10:500)

      Dimension time(0:10000)

      common/XXYY/x,y

      parameter (pltfc=1.20)  ! for plotting window

c----------
c constants
c----------

      pi  = 3.14159 265358 979323 84 D0
      pih = 0.5D0*pi
      pi2 = 2.0D0*pi
      pi4 = 4.0D0*pi
      pi8 = 8.0D0*pi

      None = 1
      Ntwo = 2
      zero = 0.0D0

c---------------------------
c read problem parameters
c from file: chan2l_exp.dat
c---------------------------

      open (1,file="chan2l_exp.dat")

       read (1,*) a
       read (1,*) beta
       read (1,*) RL
       read (1,*) yIunp
       read (1,*) aI0
       read (1,*)
       read (1,*) asrf0
       read (1,*) phisrf
       read (1,*)
       read (1,*) rho1,rho2
       read (1,*) vs1,vs2
       read (1,*)
       read (1,*) tinit
       read (1,*) cinit
       read (1,*)
       read (1,*) beta_srf
       read (1,*) Dsrf
       read (1,*)
       read (1,*) V1
       read (1,*) V2
       read (1,*) pdrop
       read (1,*) gac
       read (1,*)
       read (1,*) NSG
       read (1,*) Dt
       read (1,*) Nprint
       read (1,*) Nstep
       read (1,*)
       read (1,*) Iread
       read (1,*)
       read (1,*) method

      close (1)

c--------
c prepare
c--------

      beta = beta*pi

      sth0 = Dsin(beta)       
      cth0 = Dcos(beta)

      gx =  gac*sth0
      gy = -gac*cth0

      am = -a
      as = a**2

      vsr = vs2/vs1      ! viscosity ratio

      Drho = rho1-rho2    ! density difference
      dnr = rho2/rho1    ! density ratio 

      betac_srf = 1.0D0-beta_srf        ! surfactant
      alpha_srf = tinit*RL/(vs1*Dsrf) ! surfactant

      shiftx = -RL*cth0    ! graphics
      shifty =  RL*sth0    ! graphics

      open (2,file="chan2l_exp.xy")
      open (3,file="chan2l_exp.out")

c---
c print inclined channel walls
c---

      tmp1 =  RLm*cth0 + am*sth0
      tmp2 = -RLm*sth0 + am*cth0
      tmp3 =             am*sth0
      tmp4 =             am*cth0
      tmp5 =  RLm*cth0 + a *sth0
      tmp6 = -RLm*sth0 + a *cth0
      tmp7 =             a *sth0
      tmp8 =             a *cth0

      write (2,103) Ntwo
      write (2,102) None,tmp1,tmp2
      write (2,102) Ntwo,tmp3,tmp4

      write (2,103) Ntwo
      write (2,102) None,tmp5,tmp6
      write (2,102) Ntwo,tmp7,tmp8

c-------------------
c initial profile
c
c  Iread = 1 : Generate
c          2 : read
c-------------------

c------------------------
      if(Iread.eq.1) then        ! generate
c------------------------

        time(0) = 0.0D0

        NSGd = NSG-4
        NSGc = NSG-3
        NSGb = NSG-2
        NSGa = NSG-1

        NSG1 = NSG+1
        NSG2 = NSG+2
        NSG3 = NSG+3
        NSG4 = NSG+4

        wn = pi2/RL               ! wave number
        Dx = RL/(NSG1-1.0D0)

        Do i=-3,NSG4
          x(i) = (i-1.0D0)*Dx
          arg  = wn*x(i)
          y(i) = yIunp + aI0*cos(arg)
          arg  = wn*x(i)- phisrf*pih
          c(i) = cinit + asrf0*Dcos(arg)
        End Do

c---------
      else   ! read
c---------

        open (4,file='chan2l_exp.inp')

        read (4,*) NSG1,time(0)

        Do j=1,NSG1

          read (4,*) idle,x(j),y(j),c(j)

          x(j) = x(j)-shiftx         ! it was shifted and tilted
          y(j) = y(j)-shifty
          tmx  = x(j)*cth0 - y(j)*sth0
          tmy  = x(j)*sth0 + y(j)*cth0
          x(j) = tmx
          y(j) = tmy

          write (6,104) J,x(j),y(j),c(j)

        End Do

        close (4)
c---
c prepare
c---

        NSG  = NSG1-1
        NSGd = NSG-4
        NSGc = NSG-3
        NSGb = NSG-2
        NSGa = NSG-1

        NSG1 = NSG+1
        NSG2 = NSG+2
        NSG3 = NSG+3
        NSG4 = NSG+4

c---
c wrap
c---
        x(-3) = x(NSGc)-RL
        y(-3) = y(NSGc)
        x(-2) = x(NSGb)-RL
        y(-2) = y(NSGb)
        x(-1) = x(NSGa)-RL
        y(-1) = y(NSGa)
        x( 0) = x(NSG) -RL
        y( 0) = y(NSG)

        x(NSG2) = x(2)+RL
        y(NSG2) = y(2)
        x(NSG3) = x(3)+RL
        y(NSG3) = y(3)
        x(NSG4) = x(4)+RL
        y(NSG4) = y(4)

c-----------
      end if                  ! end of defining the initial profile
c-----------

      write (6,*) " chan2l_exp: surface nodes generated"
      write (6,*) " chan2l_exp: number of surface segments: ",NSG

c-----------------------------------------
c prepare for the finite-difference method
c-----------------------------------------

      Dx   = RL/(NSG1-1.0D0)
      Dx2  = 2.0D0*Dx
      Dx23 = 2.0D0*Dx*Dx*Dx
      Dxs  = Dx*Dx

c-----------
c initialize
c-----------

      IRKstep = 1

      Dth = 0.50D0*Dt

      Kstep  = 1   ! step counter
      Iprint = Nprint

      write (6,*) 
      write (6,*) " time, xmax,ymax, xmin, ymin"

c-.-.-.-.-.-.-.-.-.-.-.-.-.
c THE SIMULATION BEGINS HERE 
c-.-.-.-.-.-.-.-.-.-.-.-.-.

  90  Continue         ! will return here after each step

c--------------------------------------
c compute max & min interface elevation
c--------------------------------------

      call elv_max_min
     +
     +   (NSG
     +   ,xmax,ymax
     +   ,xmin,ymin
     +   )

      amp = 0.5D0*(ymax-ymin)/aI0
      amp = Dlog(amp)

      write (6,105) Kstep,time(Kstep),amp,xmax,ymax,xmin,ymin
      write (3,105) Kstep,time(Kstep),amp,xmax,ymax,xmin,ymin

      if(Kstep.eq.Nstep+1) Go to 99  ! terminate

c----------------
c surface tension
c from surfactant concentration
c----------------

 97   Continue  ! target for RK2

      Do i=-3,NSG4
       srtn(i) = tinit/betac_srf * (1.0D0-beta_srf*c(i)/cinit)
      End Do

c----------------------------
c printing session
c
c print interface profile
c so that it appears inclined
c----------------------------

      if(IRKstep.eq.1) then

      if(Iprint.eq.Nprint) then

        write (2,104) NSG1,time(Kstep)

        Do j=1,NSG1
          xpr =  x(j)*cth0 + y(j)*sth0   ! rotate
          ypr = -x(j)*sth0 + y(j)*cth0
          xpr = xpr + shiftx             ! shift
          ypr = ypr + shifty
          write (2,102) j,xpr,ypr,c(j),srtn(j)
          write (6,102) j,xpr,ypr,c(j),srtn(j)
        End Do

        Iprint = 0

      end if
      end if

c-------------------------
c compute layer thicknesses
c         spatial derivatives
c         thickness ratio
c-------------------------

      Do i=-3,NSG4
        ib = i-2
        ia = i-1
        i1 = i+1
        i2 = i+2
        DyDx1(i) = (y(i1)-y(ia))/Dx2
        DyDx2(i) = (y(i1)-2.0D0*y(i)+y(ia))/Dxs
        DyDx3(i) = (y(i2)-2.0D0*y(i1)+2.0D0*y(ia)-y(ib))/Dx23
        h1(i) = a+y(i)
        h2(i) = a-y(i)
        hr(i) = h2(i)/h1(i)
      End Do

c------------------
c compute d srtn/dx
c------------------

      Do i=1,NSG1
       DsrtnDx(i) = (srtn(i+1)-srtn(i-1))/Dx2
      End Do

c---------------
c compute dp1/dx 
c---------------

      Do i=1,NSG
       tmp = 1.0D0/(vsr+hr(i))
       Den = vsr*h1(i)**3 + h2(i)**3 + 12.0D0*vsr*tmp*as*h2(i)
       t1 = - Drho*gy*(h2(i)+6.0D0*vsr*tmp*a)*h2(i)**2 * DyDx1(i)
       t2 = - srtn(i)*(h2(i)+6.0D0*vsr*tmp*a)*h2(i)**2 * DyDx3(i)
       t3 =   rho1*gx *(vsr*h1(i)**3 + dnr*h2(i)**3
     +        +6.0D0*vsr*tmp*(1.0D0+dnr*hr(i))*a*h1(i)*h2(i) )
       t4 = 6.0D0*vs2*(V1*h1(i)+V2*h2(i))
     +    + 12.0D0*vs2*a*tmp*(vsr*V2+hr(i)*V1)
       t5 = 12.0D0*vsr*tmp*a*h2(i)*DsrtnDx(i)
       t6 = - ( 6.0D0*vsr*tmp*a + h2(i) ) * h2(i)**2 * DyDx2(i) 
     +        * DsrtnDx(i)

       dp1dx(i) = (t1+t2+t3+t4+t5+t6)/Den
       store(i) = 1.0/Den

      End Do

c--------------------------
c compute the function f(t)
c--------------------------

      sum1 = 0.0D0
      sum2 = 0.0D0

      Do i=1,NSG
        sum1 = sum1 + dp1dx(i)
        sum2 = sum2 + store(i)
      End Do

      sum1 = sum1*Dx
      sum2 = sum2*Dx

      f = (pdrop-sum1)/sum2

c-----------------------------------
c complete the computation of dp1/dx
c-----------------------------------

      Do i=1,NSG
        dp1dx(i) = dp1dx(i) + f*store(i)
      End Do

c------------
c compute the interfacial velocity uI
c------------

      Do i=1,NSG
        ri1 = 1.0D0+hr(i)
        uI(i) = a*h2(i)/vs1 *(
     +          -dp1dx(i)+rho1*gx*(1.0D0+dnr*hr(i))/ri1 )
        uI(i) = uI(i)  - 0.5D0*h2(i)**2/vs1
     +            *(Drho*gy*Dydx1(i)+srtn(i)*DyDx3(i)
     +             +DsrtnDx(i)*DyDx2(i))
        uI(i) = uI(i) + hr(i)*V1+vsr*V2
        uI(i) = uI(i) + DsrtnDx(i)*h2(i)/vs1
        uI(i) = uI(i)/(vsr+hr(i))
      End Do

c-----
c wrap
c-----

      dp1dx(0)    = dp1dx(NSG)
      dp1dx(NSG1) = dp1dx(1)

      uI( 0) = uI(NSG)
      uI(-1) = uI(NSGa)
      uI(-2) = uI(NSGb)
      uI(-3) = uI(NSGc)

      uI(NSG1) = uI(1)
      uI(NSG2) = uI(2)
      uI(NSG3) = uI(3)
      uI(NSG4) = uI(4)

c------------------------------------
c compute dQ1/dx
c
c then set dy/dt = - dQ1/dx
c------------------------------------

      pr1 = 12.0D0*Dx2*vs1
      pr2 =  2.0D0*Dx2

      pr3 = 12.0D0*Dx*vs1
      pr4 =  2.0D0*Dx

      Do i=1,NSG

       ia = i-1
       i1 = i+1

c---
       if(method.eq.1) then       ! central differences
c---

       dq1dx = ( h1(i1)**3 *(-dp1dx(i1)+rho1*gx)
     +          -h1(ia)**3 *(-dp1dx(ia)+rho1*gx))/pr1
     +     + ( (uI(i1)+V1)*h1(i1)
     +        -(uI(ia)+V1)*h1(ia))/pr2

c---
       else    ! backward differences
c---

       dq1dx = ( h1(i) **3 *(-dp1dx(i) +rho1*gx)
     +          -h1(ia)**3 *(-dp1dx(ia)+rho1*gx))/pr3
     +     + ( (uI(i) +V1)*h1(i)
     +        -(uI(ia)+V1)*h1(ia))/pr4

c---
       end if
c---

c---- alternative formulation
c
c      sum = sum - 6.0*(V1-uI(i))*DyDx1(i)
c      sum = sum - 6.0*h1(i)*(uI(i1)-uI(i1m))/Dx2
c      DyDt(i) = sum/12.0 - uI(i)*DyDx1(i)
c---------------------

       DyDt(i) = - dq1dx

      End Do


c----------------------------
c evolution of the surfactant
c----------------------------

      Do i=1,NSG
       i1 = i+1
       ia = i-1
       DcDt(i) = - (uI(i1)*c(i1)-uI(ia)*c(ia))/Dx2
     +           + Dsrf*(c(i1)-2.0D0*c(i)+c(ia))/Dxs
c      write (6,*) DcDt(i)
      End Do

c----------
c time step
c----------

c---
      if(IRKstep.eq.1) then
c---
   
      Do i=1,NSG

          ysv(i) =    y(i)
          csv(i) =    c(i)
       DyDtsv(i) = DyDt(i)
       DcDtsv(i) = DcDt(i)

       y(i) = y(i) + Dt*DyDt(i)
       c(i) = c(i) + Dt*DcDt(i)

      End Do
      
c---
      else
c---

      Do i=1,NSG
       y(i) = ysv(i) + Dth*(DyDt(i)+DyDtsv(i))
       c(i) = csv(i) + Dth*(DcDt(i)+DcDtsv(i))
      End Do

c---
      end if
c---

c-----
c wrap
c-----

      y( 0) = y(NSG)
      y(-1) = y(NSGa)
      y(-2) = y(NSGb)
      y(-3) = y(NSGc)

      y(NSG1) = y(1)
      y(NSG2) = y(2)
      y(NSG3) = y(3)
      y(NSG4) = y(4)

      c( 0) = c(NSG)
      c(-1) = c(NSGa)
      c(-2) = c(NSGb)
      c(-3) = c(NSGc)

      c(NSG1) = c(1)
      c(NSG2) = c(2)
      c(NSG3) = c(3)
      c(NSG4) = c(4)

      if(IRKstep.eq.1) then 
        IRKstep = 2
        Go to 97
      else
        IRKstep = 1
      end if

c------------------------
c reset counters and time
c------------------------

      Kstep  = Kstep+1
      Iprint = Iprint+1

      time(Kstep) = time(Kstep-1)+Dt

      Go to 90

c-.-.-.-.-.-.-.-.-.-.-.-.-
c THE SIMULATION HAS ENDED
c-.-.-.-.-.-.-.-.-.-.-.-.-

  99  Continue

c-------------------
c print restart file
c-------------------

      open (7,file="chan2l_exp.restart")  ! restart file

      write (2,104) NSG1,time(Kstep)
      write (7,104) NSG1,time(Kstep)

      Do i=1,NSG1
        xpr =  x(i)*cth0 + y(i)*sth0   ! rotate
        ypr = -x(i)*sth0 + y(i)*cth0
        xpr = xpr + shiftx             ! and shift
        ypr = ypr + shifty
        write (2,102) i,xpr,ypr
        write (7,102) i,xpr,ypr
c       write (6,102) i,xpr,ypr
      End Do
 
      write (2,104) null
      write (7,104) null

      close (7)

c----------
c finish up
c----------

      write (2,*) null

      write (2,*) 
      write (2,*) " chan2l_exp"
      write (2,*) 
      write (2,600) h
      write (2,601) beta
      write (2,602) RL
      write (2,603) yIunp
      write (2,604) aI0
      write (2,694) asrf0
      write (2,695) phss
      write (2,605) rho1,rho2
      write (2,606) vs1,vs2
      write (2,607) tinit
      write (2,697) cinit
      write (2,698) beta_srf
      write (2,699) Dsrf
      write (2,608) V1,V2
      write (2,609) pdrop
      write (2,610) gac
      write (2,611) NSG
      write (2,612) Dt
      write (2,613) Nprint
      write (2,614) Method

      close (2)

      write (3,*) null

      write (3,*) 
      write (3,*) " chan2l_exp"
      write (3,*) 
      write (3,600) h
      write (3,601) beta
      write (3,602) RL
      write (3,603) yIunp
      write (3,604) aI0
      write (3,694) a_srf
      write (3,695) phss
      write (3,605) rho1,rho2
      write (3,606) vs1,vs2
      write (3,607) tinit
      write (3,697) cinit
      write (3,698) beta_srf
      write (3,699) Dsrf
      write (3,608) V1,V2
      write (3,609) pdrop
      write (3,610) gac
      write (3,611) NSG
      write (3,612) Dt
      write (3,613) Nprint
      write (3,614) Method

      close (3)

c---
c Done
c---

 102  Format (1X,I3,20(1X,F12.8))
 103  Format (1X,I3,1X,F10.5,1X,F10.7)

 104  Format (1X,I5,1X,F12.8,1X,F12.8,1X,I1)
 105  Format (1X,I6,19(1X,F10.7))
 109  Format (" Step: ",I5," out of ",I5,"  TIME = ",f10.5)

 501  Format (4(1x,f10.5))
 502  Format (3(1x,f10.5),1x,I1)

 600  Format (' h      = ',f10.5)
 601  Format (' beta   = ',f10.5)
 602  Format (' L      = ',f10.5)
 603  Format (' Y_unp  = ',f10.5)
 604  Format (' aI0    = ',f10.5)
 694  Format (' a_srf  = ',f10.5)
 695  Format (' phss   = ',f10.5)
 605  Format (' rho1,2 = ',f10.5,1x,f10.5)
 606  Format (' vs1,2  = ',f10.5,1x,f10.5)
 607  Format (' tinit = ',f10.5)
 697  Format (' cinit = ',f10.5)
 698  Format (' beta_srf = ',f10.5)
 699  Format (' Dsrf     = ',f10.5)
 608  Format (' V1,2   = ',f10.5,1x,f10.5)
 609  Format (' pdrop  = ',f10.5)
 610  Format (' gac    = ',f10.5)
 611  Format (' NSG    = ',I3)
 612  Format (' Dt     = ',f10.5)
 613  Format (' Nprint = ',I6)
 614  Format (' Method = ',I3)

      Stop
      End
