      program airf_2d_lvp

c-----------------------------------------
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved.
c
c This program is to be used only under the
c stipulations of the licensing agreement.
c----------------------------------------

c--------------------------------------------------
c  Potential flow around a two-dimensional airfoil.
c
c  The airfoil contour is discretized into
c  vortex panels hosting a vortex sheet.
c
c  The panels are straight segments, and the
c  strength of the vortex sheet varies linearly over
c  the length each panel.
c
c  This program computes the strength of the vortex panels,
c  the distribution of the tangential velocity,
c  the pressure distribution around the airfoil,
c  and then draws streamline patterns.
c
c
c  Symbols:
c  -------
c
c  Npl: number of panels
c
c  a:   influence matrix
c  b:   velocity influence coefficients
c
c  ep:	coordinates of marker points
c  pt:	coordinates of panel end-points
c  th:  panel inclination angle
c  dl:  panel length
c
c  co:  coordinates of collocation points
c
c  cp:	pressure coefficient
c  cl:  lift coefficient
c
c  vnx, vny:  components of the unit normal vector
c  tnx, tny:  components of the unit tangential vector 
c
c  alpha: angle of attack
c
c
c  Important:
c  ---------
c  Points must be distributed in the clockckwise sense
c  starting from the trailing end and going around
c  the bottom first.
c
c---------------------------------------------------------

      Implicit Double Precision (a-h,o-z)

      Dimension ep(500,2),pt1(500,2),pt2(500,2)
      Dimension co(500,2)

      Dimension  th(500), dl(500)
      Dimension vnx(500),vny(500)
      Dimension tnx(500),tny(500)
      Dimension gamma(500),cp(500)

      Dimension a(500,500),rhs(500),xs(500)
      Dimension b(500,500)

      Dimension Npstr(100),xstr(100,4000),ystr(100,4000)

      Character kbd*1,kbd1*1            ! graphics
      Integer checkkey                  ! graphics
      Integer getkey                    ! graphics

c---
c common blocks
c---

      common/intgr1/Npl
      common/points/pt1,pt2
      common/vortex/gamma
      common/pannels/tnx,tny
      common/incident/Ux,Uy

      common/piii/pi,pi2
                           
      common/gr1/ep,co,cp                      ! graphics
      common/gr2/xstr,ystr,Nstr,Npstr          ! graphics

c----------
c constants
c----------

      pi  = 3.14159 265358D0
      pi2 = 2.0D0*pi

      null = 0

c------
c banner
c-------

      write (6,*) 
      write (6,*) ' Potential flow past an airfoil'
      write (6,*) 

c-------------------------
c Select the airfoil shape
c-------------------------

 94   Continue

      write (6,*) 
      write (6,*) ' Select the airfoil shape'
      write (6,*) 
      write (6,*) ' Enter:'
      write (6,*) 
      write (6,*) ' 0 to quit'
      write (6,*) ' 1 to read shape from file:'
      write (6,*) '   airf_2d_contour.dat'
      write (6,*) ' 2 for a NACA 4-digit section'
      write (6,*) ' 3 for a Joukowski airfoil'
      write (6,*) ' 4 for a circle'
      write (6,*) ' 5 for the NACA 23012 section'
      write (6,*) ' ----------------------------'

      read (5,*) Ishape

      If(Ishape.eq.0) Go to 99

c----------------------
c Simulation parameters
c----------------------

c     write (6,*)
c     write (6,*) ' Enter:'
c     write (6,*)
c     write (6,*) '  0 to quit'
c     write (6,*) '  1 to read them from '
c     write (6,*) '    input file: airf_2d_v_**s.dat'
c     write (6,*) '  2 to type in parameters'
c     write (6,*)
c     write (6,*) '  Warning: If you type 2 will spend a lot'
c     write (6,*) '           of time typing'
c     write (6,*) ' ---------------------------------------'
c     read  (5,*)   Ienrd

      Ienrd = 1

      If(Ienrd.eq.0) Go to 99

c-----------------------------
      If(Ienrd.eq.1) then
c-----------------------------

      If(Ishape.eq.1) then

        open (3,file='airf_2d_gnc.dat',status='old')

        read (3,*) Umag
        read (3,*) alpha
        read (3,*) chord

      Else If(Ishape.eq.2) then

        open (3,file='airf_2d_naca.dat')

        read (3,*) Umag
        read (3,*) alpha
        read (3,*) Npl
        read (3,*) thick
        read (3,*) cam
        read (3,*) xcam
        read (3,*) chord

      Else If(Ishape.eq.3) then

        open (3,file='airf_2d_jouk.dat')

        read (3,*) Umag
        read (3,*) alpha
        read (3,*) Npl
        read (3,*) xc,yc
        read (3,*) chord

      Else If(Ishape.eq.4) then

        open (3,file='airf_2d_circle.dat')

        read (3,*) Umag
        read (3,*) alpha
        read (3,*) Npl
        read (3,*) radius

      Else If(Ishape.eq.5) then

        open (3,file='airf_2d_naca_23012.dat')

        read (3,*) Umag
        read (3,*) alpha
        read (3,*) Npl
        read (3,*) thick
        read (3,*) chord

      Else 

        write (6,*)
        write (6,*)  " Sorry, this selection is not valid"
        write (6,*)

      End If

c------------------------
      Else If(Ienrd.eq.2) then
c------------------------

      write (6,*) 
      write (6,*) 'Uniform flow past a two-dimensional airfoil'
      write (6,*) '        computed by the vortex-sheet method'
      write (6,*) 
      write (6,*) 'Enter the magnitude of the incident velocity'
      write (6,*) '--------------------------------------------'
      read  (5,*) Umag

      write (6,*) 
      write (6,*) 'Enter the angle of attack (in degrees)'
      write (6,*) '--------------------------------------'
      read  (5,*) alpha

c----------------------------------
c contour specified in input file:
c
c airf_2d_contour.dat
c----------------------------------

      If(Ishape.eq.1) then

       chord = 1.0           ! for reduction only

c---------------------
c NACA 4-digit section
c---------------------

      Else If(Ishape.eq.2) then

      write (6,*) 
      write (6,*) 'Will generate a NACA 4-digit section'
  96  write (6,*) 
      write (6,*) 'Enter the number of panels; must be even'
      write (6,*) '----------------------------------------'
      read  (5,*) Npl

      If(mod(Npl,2).eq.1) then

       write (6,*)
       write (6,*) " Sorry; the number of panels must be even"
       write (6,*) " Try again"
       Go to 96

      End If

      write (6,*)
      write (6,*) 'Enter the maximum section thickness'
      write (6,*) '                      for unit cord'
      write (6,*) '-----------------------------------'
      read  (5,*) thick

      write (6,*)
      write (6,*) 'Enter the maximum section camber'
      write (6,*)                    'for unit cord'
      write (6,*) '--------------------------------'
      read  (5,*) cam

      write (6,*)
      write (6,*) 'Enter the chordwise location of max camber'
      write (6,*) '                             for unit cord'
      write (6,*)
      write (6,*) 'Must lie between 0 and 1.0'
      write (6,*) '------------------------------------------'
      read  (5,*) xcam

      write (6,*)
      write (6,*) 'Enter the chord length'
      write (6,*) '----------------------'
      read  (5,*) chord

c-------------------
c  Joukowski airfoil
c-------------------

      Else If(Ishape.eq.3) then

      write (6,*) 
      write (6,*) 'Will generate a Joukowski airfoil'
      write (6,*) '     by conformal mapping'
  97  write (6,*) 
      write (6,*) 'Enter the number of panels; must be even'
      write (6,*) '----------------------------------------'
      read  (5,*) Npl

      If(mod(Npl,2).eq.1) then

       write (6,*)
       write (6,*) " Sorry; the number of panels is not even"
       write (6,*) " Try again"
       Go to 97

      End If

      write (6,*)
      write (6,*) 'Please enter the unit circle center xc,yc'
      write (6,*)
      write (6,*) '0.2 and 0.2 are good values for testing'
      write (6,*) '----------------------------------------'
      read  (5,*) xc,yc

      write (6,*)
      write (6,*) 'Please enter the chord length'
      write (6,*)
      write (6,*) '1.0 is a good value for testing'
      write (6,*) '-------------------------------'
      read  (5,*) chord

c---------
c a circle
c---------

      Else If(Ishape.eq.4) then

      write (6,*)
      write (6,*) 'Please enter the circle radius'
      write (6,*) '------------------------------'
      read  (5,*) radius

 89   write (6,*)
      write (6,*) 'Enter number of panels; must be even'
      write (6,*) '------------------------------------'
      read  (5,*) Npl

      If(mod(Npl,2).eq.1) then

       write (6,*)
       write (6,*) " Sorry; the number of panels must be even"
       write (6,*) " Try again"
       Go to 89

      End If

c-------------------
c NACA 23012 section
c-------------------

      Else If(Ishape.eq.5) then

      write (6,*) 
      write (6,*) 'Will generate the NACA 23012 section'
  90  write (6,*) 
      write (6,*) 'Enter the number of panels; must be even'
      write (6,*) '----------------------------------------'
      read  (5,*) Npl

      If(mod(Npl,2).eq.1) then

       write (6,*)
       write (6,*) " Sorry; the number of panels must be even"
       write (6,*) " Try again"
       Go to 90

      End If

      write (6,*)
      write (6,*) 'Enter the maximum section thickness'
      write (6,*) '                      for unit cord'
      write (6,*) '-----------------------------------'
      read  (5,*) thick

      write (6,*)
      write (6,*) 'Enter the chord length'
      write (6,*) '----------------------'
      read  (5,*) chord

c-----------
      End If   ! End of options
c-----------

c-----------
      End If        ! End of reading parameters
c-----------

c---------------------------------
c  Read or generate the end-points
c---------------------------------

c-------------------------
      If(Ishape.eq.1) then
c-------------------------

      open (4,file='airf_2d_contour.dat')

       read (4,*) Npl1        ! Npl1 is the number of end-point
                              ! chord used for normalization
        Do i=1,Npl1
         read (4,*) idle,ep(i,1),ep(i,2)  ! x and y coordinates
        End Do                             ! of panel end points

        Npl = Npl1-1    ! number of panels

      close (4)

c------------------------------
      Else If(Ishape.eq.2) then             ! NACA 4-digit
c------------------------------

      Npl1  = Npl+1
      Nplh  = Npl/2
      Nplh1 = Nplh+1

      dpsi = pi2/(Npl1-1.0)

      ep(1,1) = chord
      ep(1,2) = 0.0

      Do i=2,Npl

	 psi = (i-1.0)*dpsi
	 x   = 0.5*(1.0+cos(psi))
	 
c---
c camber line
c---

	 If(x.lt.xcam) then
	  yc = cam*(2.0*xcam*x-x**2)/xcam**2
	 Else
	  yc = cam/(1.0-xcam)**2*((1.0-2.0*xcam)+2.0*xcam*x-x**2)
	 End If

c---
c thickness
c---

	 yt = 5.0*thick*
     +                  (0.2969*sqrt(x)
     +                  -0.1260*x
     +                  -0.3516*x**2
     +                  +0.2843*x**3
     +                  -0.1036*x**4
     +                  )
	
	 If(i.le.Nplh1) then
	  y = yc-yt
	 Else
	  y = yc+yt
	 End If

	 ep(i,1) = x*chord
	 ep(i,2) = y*chord
	 
      End Do

      ep(Npl1,1) = chord    ! trailing edge
      ep(Npl1,2) = 0.0      ! trailing edge

c------------------------------
      Else If(Ishape.eq.3) then     !   Joukowski airfoil
c------------------------------

      Npl1 = Npl+1

      r = sqrt((1.0-xc)**2+yc**2)

      psi  = pi2-atan(yc/(1.0-xc))
      dpsi = pi2/(Npl1-1.0)
	
      Do i=1,Npl1

	If(i.eq.1.or.i.eq.Npl1) then
	  ep(i,1) = 0.5*chord
	  ep(i,2) = 0.0
	Else
	  psi     = psi+dpsi  
          cs      = cos(psi)
          sn      = sin(psi)
	  denom   = (xc+r*cs)**2+(yc+r*sn)**2
	  ep(i,1) = (xc+r*cs)*(1.0+1.0/denom)*chord/4.0
	  ep(i,2) = (yc+r*sn)*(1.0-1.0/denom)*chord/4.0
	End If
	  
      End Do 

c------------------------------
      Else If(Ishape.eq.4) then       ! a circle
c------------------------------

      chord = 2.0*radius

      Npl1   = Npl+1
      dtheta = pi2/(Npl1-1.0)
      theta  = pi2 

      Do i=1,Npl1
       ep(i,1) = radius*cos(theta)
       ep(i,2) = radius*sin(theta)
       theta   = theta-dtheta
      End Do

c------------------------------
      Else If(Ishape.eq.5) then        ! NACA
c------------------------------

      Npl1  = Npl+1
      Nplh  = Npl/2
      Nplh1 = Nplh+1

      dpsi = pi2/(Npl1-1.0)

      ep(1,1) = chord
      ep(1,2) = 0.0

      Do i=2,Npl

	 psi = (i-1.0)*dpsi
	 x   = 0.5*(1.0+cos(psi))
	 
c---
c camber line
c---

	 If(x.lt.0.2025) then
	  yc = 2.6595*x*(0.1147+x*(-0.6075+x))
	 Else
	  yc = 0.02208*(1.0-x)
	 End If

c---
c thickness
c---

	 yt = 5.0*thick*(0.2969*sqrt(x)
     +                  -0.1260*x
     +                  -0.3516*x**2
     +                  +0.2843*x**3
     +                  -0.1036*x**4
     +                  )
	
	 If(i.le.Nplh1) then
	  y = yc-yt
	 Else
	  y = yc+yt
	 End If

	 ep(i,1) = x*chord
	 ep(i,2) = y*chord
	 
      End Do

      ep(Npl1,1) = chord    ! trailing edge
      ep(Npl1,2) = 0.0      ! trailing edge

c-----------
      End If  ! End of options
c-----------

c------------------
c open output files
c------------------

      open (2,file="airf_2d.xy",status="unknown")
      open (9,file="airf_2d.str",status="unknown")

c--------------------------------
c--------------------------------
c  End of panel generation module
c
c  Print panel end-points
c--------------------------------
c--------------------------------

      write (2,*) Npl1," panel end-nodes"
      write (9,*) Npl1," panel end-nodes"
      write (6,*) 
      write (6,*) Npl1," panel end-nodes"

      Do i=1,Npl1
	write (2,100) i,ep(i,1),ep(i,2)
	write (9,100) i,ep(i,1),ep(i,2)
	write (6,100) i,ep(i,1),ep(i,2)
      End Do

c-----------
c initialize
c-----------

      Itry = 1     ! graphics

c---
c Set coordinates of the panel end-points
c---

      Do i=1,Npl
       pt1(i,1) = ep(i,1)
       pt1(i,2) = ep(i,2)
       pt2(i,1) = ep(i+1,1)
       pt2(i,2) = ep(i+1,2)
      End Do

c---------------------------------------
c compute:
c
c  1. panel inclination angle th(j)
c     (between -pi and pi)
c  2. panel lengths
c  3. tangential and normal unit vectors
c---------------------------------------

      Do i=1,Npl
       dx     = pt2(i,1)-pt1(i,1)
       dy     = pt2(i,2)-pt1(i,2)
       dl(i)  = Dsqrt( (pt2(i,1)-pt1(i,1))**2       ! panel length
     +               + (pt2(i,2)-pt1(i,2))**2)
       th(i)  =  atan2(dy,dx)	 
       tnx(i) =  cos(th(i))     ! tangential vector
       tny(i) =  sin(th(i)) 
       vnx(i) = -tny(i)         ! normal vector points into flow
       vny(i) =  tnx(i)
      End Do

c---
c colocation points are panel mid-points
c---

      Do i=1,Npl
       co(i,1) = 0.5D0*(pt1(i,1)+pt2(i,1))
       co(i,2) = 0.5D0*(pt1(i,2)+pt2(i,2))
      End Do

c-----------------------------
c initialize the linear system
c-----------------------------

      Do i=1,Npl1
       rhs(i) = 0.0D0
       Do j=1,Npl1
        a(i,j) = 0.0D0
       End Do
      End Do

c-------------------------------------
c compute influence coefficients
c and right-hand side of linear system
c-------------------------------------

      Do i=1,Npl       ! LOOP OVER COLOCATION POINTS 
                       ! ENDS AT THE RABBIT
c---
c Compute influence of jth panel
c on ith collocation point
c---

       Do j=1,Npl     ! LOOP OVER PANELS
                      ! ENDS AT THE DOG
	 
c---
c Compute local panel coordinates (x, y).
c
c The first panel point is located at the shifted origin;
c the panel becomes horizontal
c---

        xt = co(i,1)-pt1(j,1)          ! shift colocation point
        yt = co(i,2)-pt1(j,2)

        x =  xt*tnx(j) + yt*tny(j)    ! and rotate
        y = -xt*tny(j) + yt*tnx(j)

        x1 = 0.0
        y1 = 0.0

        x2t = pt2(j,1)-pt1(j,1)         ! shift second-end point
        y2t = pt2(j,2)-pt1(j,2)

        x2  = x2t*tnx(j) + y2t*tny(j)   ! and rotate
        y2  = 0.0

c---
c Compute radial distances:  r1, r2
c         subtended angles: th1, th2
c---

        r1 = sqrt( x**2    +y**2)
        r2 = sqrt((x-x2)**2+y**2)

        th1 = atan2(y,x)
        th2 = atan2(y,x-x2)

c--------------------------------------------
c  gamma1, gamma2 are the vortex sheet strengths 
c  at the first and second panel points.
c
c  Compute influence coefficients associated with
c  the velocity components (ux, uy) 
c  corresponding to gamma1 and gamma, so that:
c
c  ux = ax1*gamma1 + ax2*gamma2
c  uy = ay1*gamma1 + ay2*gamma2
c
c  These velocities are in 
c  the jth-panel reference frame.
c
c--------------------------------------------

	If(i.eq.j) then          ! self-induced velocity

         ax1 = 0.5*(x/x2-1.0)
         ay1 = 1.0/pi2

         ax2 =-0.5*x/x2
         ay2 =-1.0/pi2

        Else

         dth  = th2-th1
         rrt  = r2/r1
         rrtl = log(rrt)
         fcc  = 1.0/(pi2*x2)

         ax1 = fcc*(     y*rrtl + (x-x2)*dth      )
         ay1 = fcc*((x-x2)*rrtl -       y*dth + x2)

         ax2 = -fcc*(y*rrtl + x*dth     )
         ay2 = -fcc*(x*rrtl - y*dth + x2)

        End If

c---
c transform the influence coefficient into the 
c global reference frame by rotation
c---
	
        ux1 = ax1*tnx(j) - ay1*tny(j)
        uy1 = ax1*tny(j) + ay1*tnx(j)

        ux2 = ax2*tnx(j) - ay2*tny(j)
        uy2 = ax2*tny(j) + ay2*tnx(j)

c-----------------------------------------------
c Compute the coefficients of gamma in the 
c master influence matrix.
c
c These are the velocity influence coefficients
c projected onto the normal vector of ith panel
c------------------------------------------------

	If(j.eq.1) then
	 a(i,1)= ux1*vnx(i) + uy1*vny(i)
	 holda = ux2*vnx(i) + uy2*vny(i)    ! hold for the next panel
	Else If(j.eq.Npl) then
	 a(i,Npl)  = ux1*vnx(i) + uy1*vny(i) + holda
	 a(i,Npl1) = ux2*vnx(i) + uy2*vny(i)
	Else
	 a(i,j)= ux1*vnx(i) + uy1*vny(i) + holda
	 holda = ux2*vnx(i) + uy2*vny(i)     ! hold for the next panel
	End If

c-------------------------------------------------------
c  b are tangential velocity influence coefficients
c
c  These are the velocity influence coefficients
c  projected onto the tangential vector of the ith panel
c-------------------------------------------------------

	If(j.eq.1) then
	 b(i,1)= ux1*tnx(i) + uy1*tny(i)
	 holdb = ux2*tnx(i) + uy2*tny(i)   ! hold for the next panel
	Else If(j.eq.Npl) then
	 b(i,Npl)  = ux1*tnx(i) + uy1*tny(i) + holdb
	 b(i,Npl1) = ux2*tnx(i) + uy2*tny(i)
	Else
	 b(i,j)= ux1*tnx(i) + uy1*tny(i) + holdb
	 holdb = ux2*tnx(i) + uy2*tny(i)   ! hold for the next panel
	End If

      End Do           ! End of loop over panels  -  DOG

      End Do          ! End of loop over collocation points -  RABBIT

c--------------------------------
c Add the Kutta condition
c expressed by the Npl+1 equation:
c
c The strength of the sheet at the first point
c of the first panel is equal and opposite
c to the strength of the sheet at the last point
c of the last panel,
c so that the mean value vanishes
c--------------------------------

      a(Npl1,1)    = 1.0
      a(Npl1,Npl1) = 1.0

c--------------------
c set right-hand side
c--------------------

  88  Continue

      al  = alpha*pi/180.0
      cal = cos(al)
      sal = sin(al)

      Ux = Umag*cal
      Uy = Umag*sal

      Do i=1,Npl
       rhs(i) = -Ux*vnx(i)-Uy*vny(i)
      End Do

c-----------------------------
c solve the linear system
c comprised of Npl+1 equations
c by Gauss elimination
c-----------------------------

      Isym_g = 0    ! the system is not symmetric
      Iwlpvt = 1    ! pivoting enabled

      call gel
     +
     +   (Npl1
     +   ,a,rhs,xs
     +   ,Isym_g
     +   ,Iwlpvt
     +   ,Det
     +   ,Istop
     +   )

c-------------------
c Display the system
c-------------------
c     Do i=1,Npl1
c      write (6,900) i,(a(i,j),j=1,Npl1),rhs(i),xs(i)
c     End Do
c-------------------

c---
c Distribute the solution
c---

      Do i=1,Npl1
       gamma(i) = xs(i)
      End Do

c---
c printing session
c---

      write (2,*) Npl1," vortex-sheet strength"
      write (6,*)
      write (6,*) " vortex-sheet strength"

      Do i=1,Npl1
       write (2,900) i,ep(i,1),gamma(i)
       write (6,900) i,ep(i,1),gamma(i)
      End Do

c---------------------------------------------
c  post-processing
c
c  compute:
c          circulation around the airfoil
c          the tangential velocity at the collocation points
c          the panel pressure coefficients a the collocation points
c          the section lift and drag coefficients
c--------------------------------------------

      write (2,*) Npl," pressure coefficient, tangential velocity"

      write (6,*)  "Pressure coefficient, tangential velocity"
      write (6,*)  
      write (6,*) Npl," pressure coefficient, tangential velocity"
      write (6,*)  

      circul  = 0.0  !  circulation
      circulg = 0.0  !  circulation in terms of gamma

      Do i=1,Npl

       velt = Ux*tnx(i)+Uy*tny(i)  ! tangential velocity of inc flow

       Do j=1,Npl1
        velt = velt + b(i,j)*gamma(j)
       End Do

       circul  = circul - velt*dl(i)                         ! circulation
       circulg = circulg+0.5*(gamma(i)+gamma(i+1))*dl(i)    ! circulation

       cp(i)  = 1.0-velt**2/Umag**2        ! pressure coefficient

       write (2,900) i,co(i,1),cp(i),velt
       write (6,900) i,co(i,1),cp(i),velt
	 
      End Do

c------------------
c print circulation
c------------------

      write (6,*)
      write (6,*) " Circulation"
      write (6,*) " computed from tangential velocity:",circul
      write (6,*)
      write (6,*) " Circulation"
      write (6,*) " computed from vortex sheet strength:",circulg
      write (6,*)

c---------------------------------------------------
c  Compute lift and drag coefficients by integration
c  of surface pressure times the panel normal vector
c---------------------------------------------------

      tlift = 0.0
      tdrag = 0.0

      Do i=1,Npl

       tlift = tlift-cp(i)*dl(i)*vny(i)
       tdrag = tdrag-cp(i)*dl(i)*vnx(i)

      End Do

c---
c normalize by chord
c---

      tlift = tlift/chord
      tdrag = tdrag/chord

c---
c Put integrated lift/drag into wind axes
c---

      wd =  tdrag*cal+tlift*sal
      wl = -tdrag*sal+tlift*cal

c---
c printing session
c---

      write (6,*) 
      write (6,*) 'Angle of attack in degrees:',alpha

      write (6,*) 
      write (6,*) 'Lift coefficient in wind axes'
      write (6,*) 'computed by integration of c_p:'
      write (6,*) 
      write (6,*) 
      write (6,*) 'Drag coefficient in wind axes'
      write (6,*) 'computed by integration of c_p:'
      write (6,*) 
      write (6,*) wd

      write (6,*) 
      write (6,*) 'Lift coefficient in wind axes'
      write (6,*) 'computed by integration of c_p:'
      write (6,*) 
      write (6,*) wl

      write (6,*) 
      write (6,*) 'Lift coefficient in wind axes'
      write (6,*) 'computed in terms of the circulation:'
      write (6,*) 
      write (6,*) -2.0*Umag*circul /chord
c     write (6,*) -2.0*Umag*circulg/chord

c--------------------------------------------
c Solution of the integral equation completed
c
c Proceed to draw streamlines
c--------------------------------------------

c---
c Read streamline data
c---

      If(Ienrd.eq.1) then

       If(Itry.eq.1) then

        read (3,*)
        read (3,*) Mstr
        read (3,*) Isc
        read (3,*) IRK
        read (3,*) stpsp

        close (3)

       End If

      Else If(Ienrd.eq.2) then

       write (6,*)
       write (6,*) "   MENU"
       write (6,*)
       write (6,*) " Enter 0 to skip streamlines"
       write (6,*) "       1 to draw streamlines"
       write (6,*) " ---------------------------"
       read  (5,*) menu

       If(menu.eq.0) Go to 999

       write (6,*)
       write (6,*) " Enter maximum number of points before "
       write (6,*) "                  stopping or inquiring"
       write (6,*) " --------------------------------------"
       read  (5,*) Mstr

       write (6,*)
       write (6,*) " Enter 0 to stop when max number is exceeded"
       write (6,*) "       1 to inquire for continuation"
       write (6,*) " -------------------------------------------"
       read  (5,*) Isc

       write (6,*) " Time integration method"
       write (6,*) 
       write (6,*) " Enter 0 to quit"
       write (6,*) "       2 to select the RK2"
       write (6,*) "       4 to select the RK2"
       write (6,*) " ------------------------ "
       read  (5,*) IRK

       If(IRK.eq.0) Go to 99

       write (6,*) 
       write (6,*) " Enter the spatial step"
       write (6,*) " ----------------------"
       read  (5,*) stpsp

      End If

c----------------------------
c Begin computing streamlines
c----------------------------

      open (4,file="streamlines.dat",status="unknown")

      read (4,*) xsmin,xsmax  ! streamline window
      read (4,*) ysmin,ysmax  ! streamline window
      read (4,*)

      Nstr = 0     ! streamline counter

  22  Continue

      Nstr=Nstr+1

      read (4,*) X0,Y0     ! stop if X0 = 99 or Y0 = 99

      If(abs(X0-99).lt.0.0000001.or.
     +   abs(Y0-99).lt.0.0000001) then
        close (4)
        Go to 999
      End If

c---

      L = 1     ! local counter for inquiry
      K = 1     ! streamline point counter

  20  Continue

      xstr(Nstr,L) = X0
      ystr(Nstr,L) = Y0

      If(X0.gt.xsmax) Go to 21
      If(X0.lt.xsmin) Go to 21
      If(Y0.gt.ysmax) Go to 21
      If(Y0.lt.ysmin) Go to 21

      call velocity
     +
     +  (X0,Y0
     +  ,Ux1,Uy1
     +  )

      step = stpsp/Dsqrt(Ux1**2+Uy1**2)       ! set time step

      Xsv = X0  ! save
      Ysv = Y0  ! save

      steph = 0.5*step

c----------------------

      If(IRK.eq.2) then

        X0 = Xsv + step * Ux1
        Y0 = Ysv + step * Uy1

        call velocity
     +                (X0,Y0
     +                ,Ux2,Uy2
     +                )
        X0 = Xsv + steph*(Ux1+Ux2)
        Y0 = Ysv + steph*(Uy1+Uy2)

      End If

c----------------------

      If(IRK.eq.4) then

        X0 = Xsv + steph * Ux1
        Y0 = Ysv + steph * Uy1

        call velocity
     +
     +       (X0,Y0
     +       ,Ux2,Uy2
     +       )

        X0 = Xsv + steph * Ux2
        Y0 = Ysv + steph * Uy2

        call velocity
     +
     +      (X0,Y0
     +      ,Ux3,Uy3
     +      )

        X0 = Xsv + step * Ux3
        Y0 = Ysv + step * Uy3

        call velocity
     +
     +     (X0,Y0
     +     ,Ux4,Uy4
     +     )

        X0 = Xsv + step * (Ux1+2.0*Ux2+2.0*Ux3+Ux4)/6.0
        Y0 = Ysv + step * (Uy1+2.0*Uy2+2.0*Uy3+Uy4)/6.0

      End If

c----------
c end of a time step
c----------

      K = K+1
      L = L+1

      If(K.lt.Mstr) Go to 20

c------
c checks
c------

      If(Isc.eq.0)  Go to 21

      K = 1    ! reset local counter

      write (6,*)
      write (6,*) "Continue this streamline ?"
      write (6,*) "                1 for yes"
      write (6,*) "-------------------------"
      read  (5,*) Icon

      If(Icon.eq.1) Go to 20

  21  Continue

      Npstr(Nstr) = L

      write (6,*) " Computed streamline :",Nstr,Npstr(Nstr)

c-------------------------
c Print out the streamline
c-------------------------

c     write (6,*)
c     write (6,*) " One streamline with ",L," points completed"
c     write (6,*)

      write (9,104) L  ! number of points

      Do I=1,L
        write (9,104) I,xstr(Nstr,I),ystr(Nstr,I)
      End Do

c----------------------------------
c Return to draw another streamline
c----------------------------------

      Go to 22

c-------------------
c End of streamlines
c-------------------

 999  Continue

c---------
c graphics
c---------

      Go to 94

c-----
c Done
c-----

 99   Continue

c     call vexit                                  ! graphics

      write (2,*) null
      close (2)
      write (9,*) null
      close (9)

 190  Format(a)
 100  Format (1x,i3,10(1x,f12.6))
 104  Format (1x,i3,20(1x,f9.5))
 900  Format (1x,i3,10(1x,f10.5))
 120  Format (90(1x,f10.5))

      Stop
      End
