      program ann2l0_dr

c-----------------------------------------
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved
c
c This program is to be used only under the
c stipulations of the licensing agreement
c----------------------------------------

c----------------------------------------------------
c  Driver for the instability of two
c  stationary annular layers 
c  confined between two coaxial cylinders,
c  in the presence of an insoluble surfactant.
c
c  Inertial effects are neglected,
c  and the motion occurs
c  under conditions of Stokes flow.
c
c  NOTATION:
c  --------
c
c  k:  disturbance wave number
c
c  a:  radius of the interface
c  ai: radius of inner cylinder
c  ae: radius of outer cylinder
c
c  visc1: viscosity of the internal fluid
c  visc2: viscosity of the external fluid
c
c  gm0:   unperturbed-state surface tension
c
c  beta:  linear constant for surface tension/surfactant conc
c         (set to zero for constant surface tension)
c
c  Ds:    surfactant diffusivity
c
c  Icase: Index for classifying configuration as follows:
c
c   1: inner cylinder present, outer cylinder present
c   2: inner cylinder absent, outer cylinder present
c   3: inner cylinder present, outer cylinder absent
c   4: inner cylinder absent, outer cylinder absent
c
c  To make the inner cylinder disappear, set ai = 0.0
c  To make the outer cylinder disappear, set ae = 0.0
c-----------------------------------------------

      Implicit Double Precision(a-h,k,o-z)

      Parameter (tol=0.000000000001)

c----------
c constants
c----------

      Null = 0

c--------------------------------
c problem parameter specification
c--------------------------------

      open (3,file="ann2l0.dat",status="unknown")

       read (3,*) a      ! interface radius
       read (3,*)
       read (3,*) ai
       read (3,*) ae
       read (3,*)
       read (3,*) visc1
       read (3,*) visc2
       read (3,*)
       read (3,*) gm0      ! unperturbed state surface tension
       read (3,*)
       read (3,*) beta
       read (3,*) Ds

      close (3)

c-----
c menu
c-----

      write (6,*) " Enter: "
      write (6,*)
      write (6,*) " 1 for one evaluation"
      write (6,*) " 2 for a graph"
      write (6,*) " 0 to quit"
      write (6,*) "--------------"
      read  (5,*) menu

      If(menu.eq.0) Go to 99

c--------
c prepare
c--------

      open (7,file="ann2l0.out")

c-------------------
c set matrix size: n
c-------------------

      if(ai.le.tol.and.ae.gt.tol) then
       Icase = 2
       n     = 6
      elseif(ai.gt.tol.and.ae.le.tol) then
       Icase = 3
       n     = 6
      elseif(ai.le.tol.and.ae.le.tol) then
       Icase = 4
       n     = 4
      else
       Icase = 1
       n     = 8
      End If

c---------------
c one evaluation
c---------------

      if(menu.eq.1) then

        write (6,*)
        write (6,*) "Please choose a wave number"
        write (6,*) "0 to quit"
        write (6,*) "---------"
        read  (5,*) k

        if(k.eq.0) Go to 99

        kstart = k
        kend   = k
        nsteps = 0
        Dk     = 0.0D0

      elseif(menu.eq.2) then

c       write (6,*)
c       write (6,*) "Please choose number of plotting intervals"
c       write (6,*) "0 to quit"
c       write (6,*) "---------"
c       read  (5,*) nsteps
        nsteps =64

        if(nsteps.eq.0) Go to 99

c       write (6,*) "Please choose kstart and kend"
c       write (6,*) "-----------------------------"
c       read  (5,*) kstart, kend
        kstart = 0.001
        kend = 1.2

        Dk = (kend-kstart)/nsteps

      End If

c-------------
c loop over k
c------------

  97  Continue

      write (6,*)
      write (6,*) " Current beta: ", beta
      write (6,*) " Please enter new beta; zero to quit"
      read  (5,*) beta
      if(beta.le.0.00000001) Go to 99

      if(kstart.lt.0.001) then
        write (6,*) " ann2l_dr: starting point readjusted"
        kstart = 0.005
      endif

      write (7,*) nsteps+1

      Do i=1,nsteps+1

       k = kstart + (i-1.0D0)*Dk

       call ann2l0 
     +
     +   (a,ai,ae
     +   ,Icase,n
     +   ,visc1,visc2,gm0
     +   ,beta,Ds
     +   ,k,gr1,gr2)

       write (6,100) i,k,gr1,gr2
       write (7,100) i,k,gr1,gr2

      End Do

      Go to 97

c-----
c Done
c-----

  99  Continue

      write (7,*) Null
      close (7)

 100  Format (1x,i3,10(1x,f15.8))

      Stop
      End
