      program ann_2l_e

c-----------------------------------------
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved
c
c This program is to be used only under the
c stipulations of the licencing agreement
c----------------------------------------

c--------------------------------------------------------
c Instability of two stationary annular layers 
c confined between two coaxial cylinders
c
c The interface has constant tension or elastic properties.
c
c NOTATION:
c --------
c
c  k:    disturbance wave number
c  a:   interface radius
c
c  ai: radius of inner cylinder
c  ae: radius of outer cylinder
c
c  visc1: viscosity of the internal fluid
c  visc2: viscosity of the external fluid
c
c  den1:  density of the internal fluid
c  den2:  density of the external fluid
c
c  gm0:   unperturbed-state surface tension
c  els:   modulus of elasticity
c
c  secm(i, j):   secular matrix
c  n:            size of the secular matrix
c
c Icase: Index for classifying configuration as follows:
c
c        1: inner cylinder present, outer cylinder present
c        2: inner cylinder absent, outer cylinder present
c        3: inner cylinder present, outer cylinder absent
c        4: inner cylinder absent, outer cylinder absent
c
c  To make the outer cylinder disappear, set ae = 0.0
c-----------------------------------------------

      Implicit Double Precision(a-h,k,o-z)

      Dimension secm(8,8)

      Parameter (tol=0.000001,Maxit= 20)  ! for Newton iterations

c---
c problem parameter specification
c---

      open(3,file="ann2lel.dat")

       read (3,*) a      ! interface radius
       read (3,*) ai     ! radius of the internal cylinder
       read (3,*) ae     ! radius of the external cylinder
       read (3,*) visc1
       read (3,*) visc2
       read (3,*) den1
       read (3,*) den2
       read (3,*) gm0      ! unperturbed state surface tension
       read (3,*) els      ! interface elasticity modulus

      close (3)

c---
c set matrix size: n
c---

      if(ai.eq.0.0.and.ae.gt.0.0) then
       Icase = 2
       n     = 6
      elseif(ai.gt.0.0.and.ae.eq.0.0) then
       Icase = 3 
       n     = 6
      elseif(ai.eq.0.0.and.ae.eq.0.0) then
       Icase = 4
       n     = 4
      else
       Icase = 1
       n     = 8   
      endif
 
c---
c inquiries
c---

  97  Continue

      write (6,*)
      write (6,*) "Please choose a wave number"
      write (6,*) "---------------------------"
      read  (5,*) k

      write (6,*)
      write (6,*) "Please enter epsilon for numerical "
      write (6,*) "differentiation in Newton iterations"
      write (6,*)
      write (6,*) "Enter 0 to quit"
      write (6,*) "---------------"
      read  (5,*) eps

      If(eps.eq.0) Go to 99

      write (6,*)
      write (6,*) "Please enter the initial guess"
      write (6,*) "for the growth rate"
      write (6,*) "------------------------------"
      read  (5,*) gr

c----
c will perform Newton iterations
c---

      write (6,*) "Iteration, growth rate, Determinant"
      Iter = 1
      Jter = 1

c---
c evaluate the secular matrix
c and its determinant
c---

  93  Continue

      call matrix
     +
     +  (gr
     +  ,a
     +  ,ai
     +  ,ae
     +  ,visc1
     +  ,visc2
     +  ,den1
     +  ,den2
     +  ,gm0
     +  ,els
     +  ,k
     +  ,Icase
     +  ,n
     +  ,secm
     +  ,det
     +  ,Istop
     +  )

      If(Istop.eq.1) Go to 99

      hold_gr  = gr
      hold_det = det

c---
c second evaluation
c---

      gr = gr+eps

      call matrix
     +
     +   (gr
     +   ,a
     +   ,ai
     +   ,ae
     +   ,visc1
     +   ,visc2
     +   ,den1
     +   ,den2
     +   ,gm0
     +   ,els
     +   ,k
     +   ,Icase
     +   ,n
     +   ,secm
     +   ,det
     +   ,Istop
     +   )

      If(Istop.eq.1) Go to 99

c---
c Newton update
c---

      slope = (hold_det-det)/(hold_gr-gr)
      diff  = -hold_det/slope
      gr    = hold_gr+diff

      write (6,100) Jter,gr,det,hold_det

c---
c stopping check
c---

      If(Iter.eq.Maxit) then
        write (6,*) " Continue the iterations?"
        write (6,*) " Enter 0 for no"
        write (6,*) " --------------"
        read  (5,*) Icon
        If(Icon.eq.0) Go to 97 
        Iter = 0
      End If

      Iter = Iter+1
      Jter = Jter+1
      If(abs(diff).gt.tol) Go to 93

c---
c return to main menu
c---

      Go to 97
c---
  99  Continue

 100  Format (1x,i3,10(1x,f20.8))

      Stop
      End
