      subroutine matrix
     +
     +   (gr
     +   ,a
     +   ,ai
     +   ,ae
     +   ,visc1
     +   ,visc2
     +   ,gm0
     +   ,els
     +   ,k
     +   ,Icase
     +   ,n
     +   ,Idet
     +   ,ar
     +   ,det
     +   ,Istop
     +   )

c-----------------------------------------
c FDLIB
c
c Copyright by C. Pozrikidis, 1999
c All rights reserved
c
c This program is to be used only under the
c stipulations of the licencing agreement
c----------------------------------------

c-----------------------------------------------
c Evaluate the secular matrix for the 
c complex phase velocity
c
c NOTATION:
c --------
c
c k:   wave number
c a:  radius of the unperturbed interface
c ai: radius of inner cylinder
c ae: radius of outer cylinder
c
c visc1: viscosity of the internal fluid
c visc2: viscosity of the external fluid
c gm0:   unperturbed-state surface tension
c els:   interfacial elasticity
c
c gr:  gr = -ikc (growth rate)

c am:   primary secular matrix for the general configuration
c       with the inner and outer cylinder present
c
c ar:   reduced secular matrix accounting for special cases
c n:    size of ar
c det:  determinant of ar
c
c Icase: Index for classifying configuration as follows:
c
c        1: inner cylinder present, outer cylinder present
c        2: inner cylinder absent, outer cylinder present
c        3: inner cylinder present, outer cylinder absent
c        4: inner cylinder absent, outer cylinder absent
c
c-----------------------------------------------
       
      Implicit Double Precision (a-h,k,o-z)

      Dimension am(8,8),ar(8,8)
      Dimension rhs(8),sol(8)    ! for Gauss elimination only

c---
c constants
c---

      pi = 3.14159 265358

c---
c prepare
c---

      ks = k*k
      as = a*a

      ka   = k*a
      kas  = ka*ka
      kasc = 1.0D0-kas

      kai = k*ai
      kae = k*ae

c------------------------------------
c Compute the modified bessel functions:
c
c  I0(k*a), I1(k*a), K0(k*a), K1(k*a)
c------------------------------------

      Iswitch = 2

      call bess_I01K01 
     +
     +    (ka,Iswitch
     +    ,BI0,BI1
     +    ,BK0,BK1
     +    )

c-------------------------------------
c compute the modified bessel functions:
c
c  I0(k*ai), I1(k*ai), K0(k*ai), K1(k*ai)
c-------------------------------------

      If(kai.gt.0.0000001) then

      call bess_I01K01 
     +     
     +   (kai,Iswitch
     +   ,BI0i,BI1i
     +   ,BK0i,BK1i
     +   )

      End If

c-------------------------------------
c compute the modified bessel functions:
c
c I0(k*ae), I1(k*ae), K0(k*ae), K1(k*ae)      
c-------------------------------------

      if(kae.gt.0.0000001) then

      call bess_I01K01 
     +
     +     (kae,Iswitch
     +     ,BI0e,BI1e
     +     ,BK0e,BK1e
     +     )

      End If

c-------------------------------------------------------
c Construct the secular matrix
c When both the inner and the outer cylinder are present,
c this is an 8 x 8 matrix
c--------------------------------------------------------
       
c----  first row:  continuity of psi at sigma = a

      am(1,1) =   BI1
      am(1,2) = a*BI0
      am(1,3) =   BK1
      am(1,4) = a*BK0
      am(1,5) =  -BI1
      am(1,6) =-a*BI0
      am(1,7) =  -BK1
      am(1,8) =-a*BK0
       
c---- second row: continuity of dpsi/dsigma at sigma = a

      am(2,1) =   k*BI0
      am(2,2) = 2.0*BI0+ka*BI1
      am(2,3) =  -k*BK0
      am(2,4) = 2.0*BK0-ka*BK1
      am(2,5) =  -k*BI0
      am(2,6) =-2.0*BI0-ka*BI1
      am(2,7) =   k*BK0
      am(2,8) =-2.0*BK0+ka*BK1
 
c---- third row:    psi = 0 at inner cylinder

      am(3,1) =    BI1i
      am(3,2) = ai*BI0i
      am(3,3) =    BK1i
      am(3,4) = ai*BK0i
      am(3,5) = 0.0
      am(3,6) = 0.0
      am(3,7) = 0.0
      am(3,8) = 0.0
       
c---- fourth row:    dpsi/dsigma = 0 at inner cylinder

      am(4,1) =   k*BI0i
      am(4,2) = 2.0*BI0i+kai*BI1i
      am(4,3) =  -k*BK0i
      am(4,4) = 2.0*BK0i-kai*BK1i
      am(4,5) = 0.0
      am(4,6) = 0.0
      am(4,7) = 0.0
      am(4,8) = 0.0
       
c---- fifth row: psi = 0  at outer cylinder

      am(5,1) = 0.0
      am(5,2) = 0.0
      am(5,3) = 0.0
      am(5,4) = 0.0 
      am(5,5) =    BI1e
      am(5,6) = ae*BI0e
      am(5,7) =    BK1e
      am(5,8) = ae*BK0e   
       
c---- sixth row: dpsi/dsigma = 0  at outer cylinder

      am(6,1) = 0.0
      am(6,2) = 0.0
      am(6,3) = 0.0
      am(6,4) = 0.0 
      am(6,5) =   k*BI0e
      am(6,6) = 2.0*BI0e+kae*BI1e
      am(6,7) =  -k*BK0e
      am(6,8) = 2.0*BK0e-kae*BK1e
       
c---- seventh row:  shear stress at interface

      am(7,1) =  2.0*visc1*BI1*ks
      am(7,2) =  2.0*visc1*(BI1+ka*BI0)*k
      am(7,3) =  2.0*visc1*BK1*ks
      am(7,4) =  2.0*visc1*(ka*BK0-BK1)*k

      am(7,5) = -2.0*visc2*BI1*ks
      am(7,6) = -2.0*visc2*(BI1+ka*BI0)*k
      am(7,7) = -2.0*visc2*BK1*ks
      am(7,8) = -2.0*visc2*(ka*BK0-BK1)*k

      fc  = -2.0*els*ks/(3.0*a*gr)

      am(7,1) = am(7,1) + fc*(-2.0*ka*BI0+BI1)
      am(7,2) = am(7,2) + fc*(-3.0*BI0-2.0*ka*BI1)*a
      am(7,3) = am(7,3) + fc*( 2.0*ka*BK0+BK1)
      am(7,4) = am(7,4) + fc*(-3.0*BK0+2.0*ka*BK1)*a

c alternative expressions
c
c     am(7,1) = am(7,1) + fc*(-2.0*a*am(2,1)+am(1,1))
c     am(7,2) = am(7,2) + fc*(-2.0*a*am(2,2)+am(1,2))
c     am(7,3) = am(7,3) + fc*(-2.0*a*am(2,3)+am(1,3))
c     am(7,4) = am(7,4) + fc*(-2.0*a*am(2,4)+am(1,4))
c

c----- eighth row:  normal stress at interface 

      am(8,1) = 2.0*visc1*ka*(ka*BI0-BI1)/as
      am(8,2) = 2.0*visc1*ka*BI1*k
      am(8,3) =-2.0*visc1*ka*(ka*BK0+BK1)/as
      am(8,4) =-2.0*visc1*ka*BK1*k

      am(8,5) =-2.0*visc2*ka*(ka*BI0-BI1)/as
      am(8,6) =-2.0*visc2*ka*BI1*k
      am(8,7) = 2.0*visc2*ka*(ka*BK0+BK1)/as
      am(8,8) = 2.0*visc2*ka*BK1*k

      fc1 = k*( -gm0*kasc+4.0*els/3.0 )/(gr*as)
      fc2 = -2.0*els*k/(3.0*a*gr)

      am(8,1) = am(8,1) + fc1*  BI1 + fc2*am(2,1)
      am(8,2) = am(8,2) + fc1*a*BI0 + fc2*am(2,2)
      am(8,3) = am(8,3) + fc1*  BK1 + fc2*am(2,3)
      am(8,4) = am(8,4) + fc1*a*BK0 + fc2*am(2,4)

c alternative expressions
c
c     am(8,1) = am(8,1) + fc1*am(1,1) + fc2*am(2,1)
c     am(8,2) = am(8,2) + fc1*am(1,2) + fc2*am(2,2)
c     am(8,3) = am(8,3) + fc1*am(1,3) + fc2*am(2,3)
c     am(8,4) = am(8,4) + fc1*am(1,4) + fc2*am(2,4)

c--------------------------------------------
c normalize to improve numerical sensitivity
c--------------------------------------------

      am(7,1) = am(7,1)*gr
      am(7,2) = am(7,2)*gr
      am(7,3) = am(7,3)*gr
      am(7,4) = am(7,4)*gr
      am(7,5) = am(7,5)*gr
      am(7,6) = am(7,6)*gr
      am(7,7) = am(7,7)*gr
      am(7,8) = am(7,8)*gr

      am(8,1) = am(8,1)*gr
      am(8,2) = am(8,2)*gr
      am(8,3) = am(8,3)*gr
      am(8,4) = am(8,4)*gr
      am(8,5) = am(8,5)*gr
      am(8,6) = am(8,6)*gr
      am(8,7) = am(8,7)*gr
      am(8,8) = am(8,8)*gr

c------------------------------------
c adjust the matrix for special cases
c------------------------------------

      if(Icase.eq.1) then

c----
c Inner cylinder present
c Outer cylinder present
c----

          Do ll=1,n
            Do jj=1,n
               ar(ll,jj)=am(ll,jj)
            End Do
          End Do

c----
c Inner cylinder absent
c Outer cylinder present
c---

      Else If(Icase.eq.2) then

          ar(1,1)=am(1,1)
          ar(1,2)=am(1,2)
          ar(1,3)=am(1,5)
          ar(1,4)=am(1,6)
          ar(1,5)=am(1,7)
          ar(1,6)=am(1,8)

          ar(2,1)=am(2,1)
          ar(2,2)=am(2,2)
          ar(2,3)=am(2,5)
          ar(2,4)=am(2,6)
          ar(2,5)=am(2,7)
          ar(2,6)=am(2,8)

          ar(3,1)=am(5,1)
          ar(3,2)=am(5,2)
          ar(3,3)=am(5,5)
          ar(3,4)=am(5,6)
          ar(3,5)=am(5,7)
          ar(3,6)=am(5,8)

          ar(4,1)=am(6,1)
          ar(4,2)=am(6,2)
          ar(4,3)=am(6,5)
          ar(4,4)=am(6,6)
          ar(4,5)=am(6,7)
          ar(4,6)=am(6,8)

          ar(5,1)=am(7,1)
          ar(5,2)=am(7,2)
          ar(5,3)=am(7,5)
          ar(5,4)=am(7,6)
          ar(5,5)=am(7,7)
          ar(5,6)=am(7,8)

          ar(6,1)=am(8,1)
          ar(6,2)=am(8,2)
          ar(6,3)=am(8,5)
          ar(6,4)=am(8,6)
          ar(6,5)=am(8,7)
          ar(6,6)=am(8,8)

c----
c Inner cylinder present
c Outer cylinder absent
c----

       Else If(Icase.eq.3) then

          ar(1,1)=am(1,1)
          ar(1,2)=am(1,2)
          ar(1,3)=am(1,3)
          ar(1,4)=am(1,4)
          ar(1,5)=am(1,7)
          ar(1,6)=am(1,8)

          ar(2,1)=am(2,1)
          ar(2,2)=am(2,2)
          ar(2,3)=am(2,3)
          ar(2,4)=am(2,4)
          ar(2,5)=am(2,7)
          ar(2,6)=am(2,8)

          ar(3,1)=am(3,1)
          ar(3,2)=am(3,2)
          ar(3,3)=am(3,3)
          ar(3,4)=am(3,4)
          ar(3,5)=am(3,7)
          ar(3,6)=am(3,8)

          ar(4,1)=am(4,1)
          ar(4,2)=am(4,2)
          ar(4,3)=am(4,3)
          ar(4,4)=am(4,4)
          ar(4,5)=am(4,7)
          ar(4,6)=am(4,8)

          ar(5,1)=am(7,1)
          ar(5,2)=am(7,2)
          ar(5,3)=am(7,3)
          ar(5,4)=am(7,4)
          ar(5,5)=am(7,7)
          ar(5,6)=am(7,8)

          ar(6,1)=am(8,1)
          ar(6,2)=am(8,2)
          ar(6,3)=am(8,3)
          ar(6,4)=am(8,4)
          ar(6,5)=am(8,7)
          ar(6,6)=am(8,8)  

c----
c Inner cylinder absent
c Outer cylinder absent
c----

       Else If(Icase.eq.4) then

          ar(1,1)=am(1,1)
          ar(1,2)=am(1,2)
          ar(1,3)=am(1,7)
          ar(1,4)=am(1,8)

          ar(2,1)=am(2,1)
          ar(2,2)=am(2,2)
          ar(2,3)=am(2,7)
          ar(2,4)=am(2,8)

          ar(3,1)=am(7,1)
          ar(3,2)=am(7,2)
          ar(3,3)=am(7,7)
          ar(3,4)=am(7,8)

          ar(4,1)=am(8,1)
          ar(4,2)=am(8,2)
          ar(4,3)=am(8,7)
          ar(4,4)=am(8,8)
           
      End If
       
c---
c printing
c---
c
c     Do i=1,n
c      write (6,100) (ar(i,j),j=1,n)
c     End Do

c------------------------------
c compute the determinant of ar
c but only if Idet = 1
c------------------------------

      if(Idet.eq.1) then

      Isym_gel = 0   ! matrix is not symmetric
      Iwlpvt   = 1   ! pivoting enabled

      call gel
     +
     +  (n,ar
     +  ,rhs
     +  ,sol
     +  ,Isym_gel
     +  ,Iwlpvt
c    +  ,l,u
     +  ,det
     +  ,Istop
     +  )

      End If

c---
c done
c---

 100  Format (10(1x,f10.5))

      Return
      End
