close all
clear all

%-------------------------------------------------
%  Normal mode analysis of unidirectional inviscid
%  shear flow along the x axis
%
%  The flow is confined between two parallel walls
%  located at y = a and -b
%
%  SYMBOLS:
%  -------
%
%  U:   velocity, function of y
%  Upp:	second derivative of U with respect to y
%  N:   level of discretization of y axis within (a,b)
%
%  kdiv:  number of k divisions
%         will span the wave-number range (kmin, kmax)
%
%  epsr: real step for Newton corrections
%  epsi: imag step for Newton corrections
%  tole: tolerance for convergence	
%
%  cr, ci:	real and imaginary parts of c
%---------------------------------------------

a = 5.0;
b = 5.0;
menu = 1;
U0 = 1.0;
delta = 1.0;
alpha = 1.0;

N = 2*2*64;
N = 128;

kstart = 0.01;
kend = 2.0;
kdiv = 64;
newter = 20;  %  max Newton iterations
epsr = 0.01;  % for Newton iterations
epsi = 0.01; % for Newton iterations
tole = 0.000001;  % for stopping Newton iterations

cr = 0.7; % initial guess
ci = 0.1; % initial guess

%--------
% prepare
%--------

denn = epsr^2 + epsi^2;
Dk = (kend-kstart)/kdiv;
Dy = (a+b)/N;
Dys = Dy^2;

figure(1)
hold on
set(gca,'fontsize',15)
xlabel('k\delta','fontsize',15)
ylabel('c_I/U_0','fontsize',15)
axis([0 1 0 0.8])
box on

figure(2)
hold on
set(gca,'fontsize',15)
xlabel('k\delta','fontsize',15)
ylabel('\sigma\delta/U_0','fontsize',15)
axis([0 1 0 0.2])
box on

figure(3)
hold on
set(gca,'fontsize',15)
xlabel('k \delta','fontsize',15)
ylabel('r_R','fontsize',15)
box on

%----------
% velocity profile
%----------

for i=1:N+1

  y = -b+(i-1)*Dy;

  [U(i),Up(i),Upp(i)] = rayleigh_vel(menu,U0,delta,alpha,y);

  % compute the second derivative numerically (redundant)

  Ido = 0;
  if(Ido==1)
   epsU  = 0.00001;  % for computing U''
   epsUs  = epsU^2;
   [U1,Up1,Upp1] = rayleigh_vel(menu,U0,delta,alpha,y+epsU);
   [U2,Up2,Upp2] = rayleigh_vel(menu,U0,delta,alpha,y-epsU);
   Upp_num(i) = (U1-2.0*U(i)+U2)/epsUs;
   Upp(i) = Upp_num(i);
  end

end

%--------------------------------------------------
% scan the wave-number axis from k = kstart to kend
%--------------------------------------------------

%=======================
  for kloop=1:kdiv+1    % scan the wave number axis
%=======================

  k = kstart+(kloop-1)*Dk;

%---
%  Newton iterations
%---

  %---
  for iter=1:newter
  %---

  [detr, deti] = rayleigh_sys (N,Dys,U,Upp,k,cr,ci);

  cr = cr+epsr;
  ci = ci+epsi;

  [detr1, deti1] = rayleigh_sys (N,Dys,U,Upp,k,cr,ci);

% [detr,deti]
% [detr1,deti1]
% pause

  cr = cr-epsr;
  ci = ci-epsi;

%---
% compute the derivative DdetDc = d(det(E))/dc
%---

  Ddetr = detr1-detr;
  Ddeti = deti1-deti;
  DdetDcr = ( Ddetr*epsr + Ddeti*epsi)/denn;
  DdetDci = (-Ddetr*epsi + Ddeti*epsr)/denn;

% [DdetDcr,DdetDci]
% pause

%---
% compute the correction
%---

  den = DdetDcr^2+DdetDci^2;

  corrr = ( detr*DdetDcr + deti*DdetDci)/den;
  corri = (-detr*DdetDci + deti*DdetDcr)/den;

%---
% make the correction
%---

  cr = cr - corrr;
  ci = ci - corri;

  corr = sqrt(corrr^2+corri^2);

  if(corr<tole) 
   break
  end

%---
  end  % of Newton iterations
%---

%---
% printing session
%---

 sigma = ci*k;

 [k,cr,ci,sigma]

 kplot(kloop) = k;
 crplot(kloop) = cr;
 ciplot(kloop) = ci;
 sigmaplot(kloop) = sigma;

 if(ci<0.001)
  break
 end

%===
  end % of scanning the wave number
%===

figure(1)
plot(kplot,ciplot,'k-','linewidth',1)

figure(2)
plot(kplot,sigmaplot,'k-','linewidth',1)

figure(3)
plot(kplot,crplot,'k-','linewidth',1)

clear kplot ciplot sigmaplot crplot
